#! /usr/bin/env python
import gobject
import gtk
import bonobo
import CORBA
import textview_display as disp
#import gtkhtml2_display as disp
import entry
import varbrowser
import numexp_client
import gc
import os.path
import Numexp
import locale
import gconf
import gnomevfs
from modules_window import ModulesWindow

VIEW_DEFINITIONS_TOGGLE_KEY = "/apps/gnumexp/Console/show_definitions"
MATHML_OUTPUT_TOGGLE_KEY    = "/apps/gnumexp/Console/mathml_output"
SIMPLIFICATION_TOGGLE_KEY    = "/apps/gnumexp/Console/simplification"


ui_info = \
"""
<ui>
  <menubar name='MenuBar'>
    <menu action='ViewMenu'>
      <menuitem action='Clear'/>
      <separator/>
      <menuitem action='Definitions'/>
      <menuitem action='MathMLOutput'/>
      <menuitem action='Simplification'/>
      <separator/>
      <menuitem action='Modules'/>
    </menu>
  </menubar>
  <toolbar name='ToolBar'>
    <toolitem action='Definitions'/>
    <toolitem action='Clear'/>
  </toolbar>
</ui>
"""

class Console(object):

    def __init__(self, main_window):
        self.topic_list = None
        conf = gconf.client_get_default()
        self._vpaned = gtk.VPaned()
        self._vpaned.show()
        self._hpaned = gtk.HPaned()
        self._hpaned.show()

        self.display = disp.Display()

        self._vpaned.pack1(self.display.get_widget(), True, False)

        self.entry = entry.Entry()
        self.entry.widget.show()

        frame = gtk.Frame()
        frame.set_shadow_type(gtk.SHADOW_IN)
        frame.show()
        frame.add(self.entry.widget)
        self._vpaned.pack2(frame, False, False)

        self._hpaned.pack1(self._vpaned, True, False)
        self._varbrowser = varbrowser.View(varbrowser.Model(), self)
        self._hpaned.pack2(self._varbrowser.widget, True, False)
        try:
            self.entry.history.load(file(os.path.expanduser("~/.gnumexp_history")))
        except IOError:
            pass
        self.entry.connect("text-entered", self.__entry_text_entered)
        self.entry.connect("completions-request", self.__entry_completions_request)
        self.kernel_wrapper = main_window.kernel
        self.kernel = self.kernel_wrapper.kernel
        self._varbrowser.model.update_namespace("::", self.kernel)

        self.client = numexp_client.NumexpClient(self, self.kernel)
        self.kernel.setClient(self.client._this())
        self.kernel.registerNamespaceListener("::")

        self.entry.grab_focus()
        self.widget.connect("destroy", self.__destroy)

        self.entry.widget.connect("key-press-event", self.__entry_key_press)

        ## Read GConf settings
        self.__mathml_output = conf.get_bool(MATHML_OUTPUT_TOGGLE_KEY)
        self.simplification = conf.get_bool(SIMPLIFICATION_TOGGLE_KEY)
        self._varbrowser.widget.set_property("visible", conf.get_bool(VIEW_DEFINITIONS_TOGGLE_KEY))

        ## Merge console UI elements
        mergeid = main_window.ui_manager.add_ui_from_string(ui_info)
        main_window.ui_manager.insert_action_group(self.__create_action_group(), 0)
        self._add_topic_list(main_window.ui_manager)

    def __create_action_group(self):
        # GtkActionEntry
        entries = (
          ( "ViewMenu", None, _("_View") ),                # name, stock id, label

          ( "Clear", gtk.STOCK_CLEAR,              # name, stock id
            None, "<control>L",                        # label, accelerator
            _("Clear console contents"),               # tooltip
            self.on_clear),

          ( "Modules", None,                       # name, stock id
            "Show _Modules", "<control>M",             # label, accelerator
            _("Show list of available modules"),       # tooltip
            self.on_show_modules),

        );

        # GtkToggleActionEntry
        toggle_entries = (
          ( "MathMLOutput", None,                              # name, stock id
            _("_Pretty Output"), "<control>N",                 # label, accelerator
            _("Toggle whether to use a more elaborate expression renderer,"
              " or simply show output as text"),                       # tooltip
            self.on_toggle_mathml_output, self.__mathml_output),

          ( "Definitions", "gnumexp-definitions",        # name, stock id
            _("_Definitions"), "F9",                     # label, accelerator
            _("Show or hide the definitions side pane"), # tooltip
            self.on_toggle_definitions, self._varbrowser.widget.props.visible),

          ( "Simplification", None,                                   # name, stock id
            _("_Simplification"), None,                               # label, accelerator
            _("Enable result simplification instead of evaluation"),  # tooltip
            self.on_toggle_simplification, self.simplification),
        )

        # Create the menubar and toolbar
        action_group = gtk.ActionGroup("ConsoleActions")
        action_group.add_actions(entries)
        action_group.add_toggle_actions(toggle_entries)

        return action_group


    def __entry_key_press(self, widget, event):
        if event.state == 0 and event.keyval == gtk.keysyms.Page_Up:
            self.display.page_up()
        elif event.state == 0 and event.keyval == gtk.keysyms.Page_Down:
            self.display.page_down()
        else:
            return False
        return True
    
    def __destroy(self, *args):
        self.client.unref()
        self.kernel.unref()
        self.entry.history.save(file(os.path.expanduser("~/.gnumexp_history"), "w"))

    def set_transient_for(self, win):
        win.set_transient_for(self.widget.get_toplevel())
            
    def _eval_user_input(self, text):
        tree = self.kernel_wrapper.Element(self.kernel, self.kernel.parseString(text))
        if self.simplification:
            result = self.kernel_wrapper.Element(self.kernel, self.kernel.elementSimplify(tree.id))
        else:
            result = self.kernel_wrapper.Element(self.kernel, self.kernel.elementEval(tree.id))
        self.kernel.defineVariable("::ans", result.id)
        return self.kernel.elementStr(result.id)

    def _eval_user_input_mathml(self, text):
        tree = self.kernel_wrapper.Element(self.kernel, self.kernel.parseString(text))
        input_mathml = self.kernel.elementToMathML(tree.id)
        if self.simplification:
            result = self.kernel_wrapper.Element(self.kernel, self.kernel.elementSimplify(tree.id))
        else:
            result = self.kernel_wrapper.Element(self.kernel, self.kernel.elementEval(tree.id))
        self.kernel.defineVariable("::ans", result.id)
        output_mathml = self.kernel.elementToMathML(result.id)
        return (input_mathml, output_mathml)

    def __entry_text_entered(self, entry, text):
        if text:
            self.read_eval_print(text)

    def read_eval_print(self, text):
        try:
            if self.__mathml_output:
                input_mathml, output_mathml =\
                	      self._eval_user_input_mathml(text)
            else:
                result = self._eval_user_input(text)
        except CORBA.UserException, ex:
            self.samples = None
            err = str(ex.__class__) + ": " + ex.detail
            dialog = gtk.MessageDialog(
                flags          = gtk.DIALOG_DESTROY_WITH_PARENT,
                type           = gtk.MESSAGE_ERROR,
                buttons        = gtk.BUTTONS_OK,
                message_format = err)
            dialog.set_property("has-separator", gtk.FALSE)
            self.set_transient_for(dialog)
            dialog.run()
            dialog.destroy()
            return
        if self.__mathml_output:
            self.display.display_qa_mathml(
                "<math display='block'>%s</math>" % input_mathml,
                "<math display='block'>%s</math>" % output_mathml)
        else:
            self.display.display_qa(text, result)

    def get_widget(self):
        return self._hpaned

    widget = property(get_widget, None, None,
                      "return the entry toplevel container widget")

    def display_help_on(self, symbol):
        #print "display_help_on(\"%s\")" % symbol
        info = self.kernel.getDefinitionInfo(symbol)
        self.display.display_help(symbol, info.description)

    def __entry_completions_request(self, entry, prefix):
        # convert relative paths to absolute ones, considering the cwd
        if len(prefix) < 2 or prefix[:2] != '::':
            prefix = self.client.cwd + prefix
            need_strip_root = True
        else:
            need_strip_root = False
        
        namespace = prefix[:prefix.rfind("::") + 2]
        #print "completing using namespace ", namespace
        try:
            completions = self._varbrowser.model.get_namespace_listing(namespace)
        except ValueError:
            return None
        #print "completions:", completions
        
        if need_strip_root:
            completions = [comp[2:] for comp in completions if comp.startswith(prefix)]
        else:
            completions = [comp for comp in completions if comp.startswith(prefix)]
        completions.sort(locale.strcoll)
        return completions

    def on_toggle_definitions(self, action):
        self._varbrowser.widget.set_property("visible", action.get_active())
        gconf.client_get_default().set_bool(VIEW_DEFINITIONS_TOGGLE_KEY, action.get_active())

    def on_toggle_mathml_output(self, action):
        self.__mathml_output = action.get_active()
        gconf.client_get_default().set_bool(MATHML_OUTPUT_TOGGLE_KEY, action.get_active())

    def on_clear(self, action):
        self.display.clear()

    def on_toggle_simplification(self, action):
        self.simplification = action.get_active()
        gconf.client_get_default().set_bool(SIMPLIFICATION_TOGGLE_KEY, action.get_active())

    def on_show_modules(self, action):
        modwin = ModulesWindow(self.kernel_wrapper)
        self.set_transient_for(modwin.window)
        modwin.window.show_all()
        modwin.window.connect("response", lambda win, resp: win.destroy())

    def on_topic_help(self, action):
        index = int(action.props.name.split("TopicHelp")[1])
        topic = self.topic_list[index]
        body = self.kernel.getTopicHelp(topic)
        self.display.display_help(topic, body)

    def _add_topic_list(self, ui_manager):
        topicXML = ["<ui>\n<menubar name='MenuBar'><menu action='HelpMenu'><placeholder name='TopicListAdditions'>\n"]
        self.topic_list = self.kernel.getTopicList()
        actions = []
        for i, topic in enumerate(self.topic_list):
            actions.append(
                ( "TopicHelp%i" % i, None,                       # name, stock id
                  topic, None,                                   # label, accelerator
                  _("Display help on the topic \"%s\"") % topic, # tooltip
                  self.on_topic_help))
            topicXML.append("  <menuitem action=\"TopicHelp%i\"/>" % i)
        topicXML.append("<separator/></placeholder></menu></menubar></ui>")
        mergeid = ui_manager.add_ui_from_string("\n".join(topicXML))
        action_group = gtk.ActionGroup("ConsoleTopicHelpActions")
        action_group.add_actions(actions)
        ui_manager.insert_action_group(action_group, 0)

    def load_from_uri(self, uri):
        try:
            file_name = gnomevfs.get_local_path_from_uri(uri)
        except RuntimeError:
            dialog = gtk.MessageDialog(type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK,
                                       message_format=_("Unable to load remote scripts"))
            dialog.run()
            dialog.destroy()
            return
        assert '"' not in file_name
        self.read_eval_print("load[\"%s\"]" % (file_name,))

