#! /usr/bin/env python
import sys, os
import bonobo.ui, gconf, Bonobo
import gnumexp_conf
import gtk.gdk
import console
from GNumexp_Common import util
from modules_window import ModulesWindow

icon = gtk.gdk.pixbuf_new_from_file(os.path.join(gnumexp_conf.ICONDIR, "gnumexp_icon.png"))
gtk.window_set_default_icon_list(icon); del icon

VIEW_DEFINITIONS_TOGGLE_KEY = "/apps/gnumexp/Console/show_definitions"
MATHML_OUTPUT_TOGGLE_KEY    = "/apps/gnumexp/Console/mathml_output"
SIMPLIFICATION_TOGGLE_KEY    = "/apps/gnumexp/Console/simplification"

class ConsoleControl(bonobo.ui.Control):
    def __init__(self):
        self.console = console.Console()
        self.console.control = self
        bonobo.ui.Control.__init__(self, self.console.widget)
        self.connect('set-frame', self.__set_frame_cb)

    def __set_frame_cb(self, control):
        frame = control.get_control_frame()
        if frame is None:
            uic = control.get_ui_component()
            uic.unset_container()
        else:
            ui_component = self.get_ui_component()
            ui_container = self.get_remote_ui_container()
            ui_component.set_container(ui_container)
            bonobo.ui.util_set_ui(ui_component, '',
                		  gnumexp_conf.UIDIR + '/Numexp_Console-ui.xml',
                		  'numexp-console')
            verbs = [
                ("ViewDefinitions", self.__view_definitions_toggle_cb),
                ("Modules",         self.__modules_cb),
                ("MathMLOutput",    self.__mathml_output_toggle_cb),
                ("Clear",           self.__clear_cb),
                ("Simplification",  self.__simplification_toggle_cb),
                ]
            ui_component.add_verb_list(verbs)
            ui_component.connect('ui-event', self.__ui_event)
            self._read_settings(ui_component)
            self.__add_topic_list(ui_container)
            ui_container.unref()

    def _read_settings(self, uic):
        """Set the initial toggle states from GConf settings"""
        gc = gconf.client_get_default()

        # Variables side pane toggle
        val = gc.get(VIEW_DEFINITIONS_TOGGLE_KEY)
        if val is not None:
            state = val.get_bool()
            val = util.bool_str(state)
        else:
            state = True
            val = "1"
        uic.set_prop("/commands/ViewDefinitions", "state", val)
        self.console.set_varbrowser_active(state)

        # MathML Output toggle
        val = gc.get(MATHML_OUTPUT_TOGGLE_KEY)
        if val is not None:
            state = val.get_bool()
            val = util.bool_str(state)
        else:
            state = True
            val = "1"
        uic.set_prop("/commands/MathMLOutput", "state", val)
        self.console.set_mathml_output(state)

        # Simplification toggle
        val = gc.get(SIMPLIFICATION_TOGGLE_KEY)
        if val is not None:
            state = val.get_bool()
            val = util.bool_str(state)
        else:
            state = True
            val = "1"
        uic.set_prop("/commands/Simplification", "state", val)
        self.console.set_simplification(state)

    def __view_definitions_toggle_cb(self, ui_component, id): pass
    def __mathml_output_toggle_cb(self, ui_component, id): pass
    def __simplification_toggle_cb(self, ui_component, id): pass

    def __ui_event(self, ui_component, id, type, state):
        if type == Bonobo.UIComponent.STATE_CHANGED:
            if id == "ViewDefinitions":
                state = util.parse_bool(state)
                self.console.set_varbrowser_active(state)
                gconf.client_get_default().set_bool(VIEW_DEFINITIONS_TOGGLE_KEY, state)
            elif id == "MathMLOutput":
                state = util.parse_bool(state)
                self.console.set_mathml_output(state)
                gconf.client_get_default().set_bool(MATHML_OUTPUT_TOGGLE_KEY, state)
            elif id == "Simplification":
                state = util.parse_bool(state)
                self.console.set_simplification(state)
                gconf.client_get_default().set_bool(SIMPLIFICATION_TOGGLE_KEY, state)

    def __modules_cb(self, ui_component, id):
        modwin = ModulesWindow(self.console.kernel)
        self.set_transient_for(modwin.window)
        modwin.window.show_all()
        modwin.window.connect("response", lambda win, resp: win.destroy())

    def __clear_cb(self, ui_component, id):
        self.console.clear()

    def __verb_TopicHelp(self, uic, cname, topic,):
        body = self.console.kernel.getTopicHelp(topic)
        self.console.display.display_help(topic, body)

    def __add_topic_list(self, ui_container):
        component = bonobo.ui.Component("HelpTopicList")
        component.set_container(ui_container)
        topicXML = ["<Root>\n<menu>\n<submenu name=\"Help\">"]
        topic_list = self.console.kernel.getTopicList()
        for i, topic in enumerate(topic_list):
            topicXML.append("  <menuitem name=\"TopicHelp%i\" "
                            "verb=\"TopicHelp%i\" label=\"%s\"/>" %
                            (i, i, topic))
            component.add_verb("TopicHelp%i" % i, self.__verb_TopicHelp, topic);
        topicXML.append("<separator/></submenu></menu></Root>")
        component.set_translate("/", "\n".join(topicXML))
        component.unref()

