#! /usr/bin/env python
import pygtk; pygtk.require("2.0")
import gobject, gtk
import string
import pango


class CompletionSelector(gobject.GObject):

    __gsignals__ = {
        "completion": (gobject.SIGNAL_RUN_LAST, # signal flags
                       None, # return type
                       [str] # param types
                       )
        }

    def __init__(self, completions):
        self.__gobject_init__()
        self.window = gtk.Window(gtk.WINDOW_POPUP)
        frame = gobject.new(gtk.Frame,
                	    shadow=gtk.SHADOW_ETCHED_OUT,
                	    visible=True)
        self.window.add(frame)
        sw = gtk.ScrolledWindow()
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        sw.show()
        frame.add(sw)

        liststore = gtk.ListStore(gobject.TYPE_STRING)
        for comp in completions:
            iter = liststore.append()
            liststore.set(iter, 0, comp)
            
        self.treeview = gtk.TreeView(liststore)
        self.treeview.modify_font(pango.FontDescription("Monospace"))
        self.treeview.set_property("headers-visible", False)
        renderer = gtk.CellRendererText()
        renderer.set_property("xalign", 0.0)
        column = self.treeview.insert_column_with_attributes(
            -1, "Completions", renderer, text=0)
        sw.add(self.treeview)
        self.treeview.show()
        self.window.connect("key-press-event", self.__key_press)
        self.treeview.connect("row_activated", self.__row_activated)

    def __key_press(self, widget, event):
        if event.state == 0 and event.keyval == gtk.keysyms.Escape:
            self.window.destroy()
        
    def __row_activated(self, tree, path, column):
        model = tree.get_model()
        iter = model.get_iter(path)
        if iter is None: return
        completion = model.get_value(iter, 0)
        self.emit("completion", completion)
        self.window.destroy()
# class_init
# gobject.type_register(CompletionSelector)


# -------------------------------------------------
class History(object):
    def __init__(self):
        self.last_op_was_an_add = False
        self.history = []
        self.pos = None

    def next(self, histdata):
        if self.pos is None: return
        if self.history:
            if self.pos < len(self.history) - 1:
                self.history[self.pos] = histdata
                self.pos += 1
                return self.history[self.pos]

    def previous(self, histdata):
        if self.history:
            if self.pos is None:
                self.pos = len(self.history)
                self.history.append(histdata)
            self.history[self.pos] = histdata
            self.pos -= 1
            if self.pos < 0:
                self.pos = 0
                return
            return self.history[self.pos]

    def add(self, histdata):
        if histdata[0]:
            self.history.append(histdata)
            self.pos = None

    def load(self, file):
        self.history = []
        for l in file:
            l = l.rstrip()
            self.history.append((l, len(l)))

    def save(self, file):
        for f in self.history:
            print >> file, f[0]
            
# ----------------------------------------
class Entry(gobject.GObject):

    _word_separators = (string.punctuation + string.whitespace).replace(":", "")

    __gsignals__ = {
        "text-entered": (gobject.SIGNAL_RUN_LAST, # signal flags
                	 None, # return type
                	 (str,) # param types
                	 ),
        "completions-request": (gobject.SIGNAL_RUN_LAST, # signal flags
                		object, # return type: list of completions
                		[str] # param types: completion prefix
                		),
        }
    
    def __init__(self):
        self.__gobject_init__()
        self.history = History()
        self.entry = gtk.Entry()
        self.entry.modify_font(pango.FontDescription("Monospace"))
        self.entry.connect("activate", self.__entry_activate)
        self.entry.connect("key-press-event", self.__entry_key_press_event)

    def __entry_activate(self, entry):
        hist = self.__get_history_data()
        text = entry.get_text()
        entry.set_text("")
        self.history.add(hist)
        self.emit("text-entered", text)

    def get_widget(self):
        "return the entry toplevel container widget"
        return self.entry

    def grab_focus(self):
        self.entry.grab_focus()

    widget = property(get_widget, None, None, None)

    def __get_history_data(self):
        return self.entry.get_text(), self.entry.get_position()
    def __set_history_data(self, data):
        self.entry.set_text(data[0])
        self.entry.set_position(data[1])

    def previous(self):
        histdata = self.history.previous(self.__get_history_data())
        if histdata is not None:
            self.__set_history_data(histdata)
        else:
            gtk.gdk.beep()

    def next(self):
        histdata = self.history.next(self.__get_history_data())
        if histdata is not None:
            self.__set_history_data(histdata)
        else:
            gtk.gdk.beep()
        

    def __entry_key_press_event(self, entry, event):
        #if event.keyval == gtk.keysyms.space and event.state & gtk.gdk.CONTROL_MASK:
        #    console_entry_toggle_expanded(wid, self);
        #    return True

        modifiers = gtk.accelerator_get_default_mod_mask ()
        if (event.state & modifiers) == 0:
            if event.keyval == gtk.keysyms.Up:
                self.previous()
                return True
            elif event.keyval == gtk.keysyms.Down:
                self.next()
                return True
            elif event.keyval == gtk.keysyms.Tab:
                self.__tab_handler()
                return True
        elif (event.state & modifiers) == gtk.gdk.CONTROL_MASK:
            if event.keyval == gtk.keysyms.p:
                self.previous()
                return True
            elif event.keyval == gtk.keysyms.n:
                self.next()
                return True
        return False

    def insert_text_at_cursor(self, text):
        pos = self.entry.get_position()
        self.entry.insert_text(text, pos)
        self.entry.set_position(pos + len(text))

    def grab_focus(self):
        pos = self.entry.get_position()
        self.entry.grab_focus()
        self.entry.select_region(0, 0)
        self.entry.set_position(pos)

    def _complete_with(self, completion):
        remaining = completion[len(self._completion_prefix):]
        self.insert_text_at_cursor(remaining)

    def __completion_selected(self, comp, text):
        self._complete_with(text)

    def __tab_handler(self):
        text = self.entry.get_text().decode("utf-8")
        pos  = self.entry.get_position()
        start = None
        for i in range(pos - 1, -1, -1):
            if text[i] in self._word_separators:
                start = i + 1
                break
        if start is None:
            start = 0
        self._completion_prefix = text[start:pos]
        self._completion_start  = start
        self._completion_end    = pos
        #if len(self._completion_prefix) < 3:
        #    gtk.gdk.beep()
        #    return
        alternatives = self.emit("completions-request", self._completion_prefix)
        if alternatives is None or not isinstance(alternatives, list):
            gtk.gdk.beep()
            return
        elif len(alternatives) == 1:
            self._complete_with(alternatives[0])
            return
        elif len(alternatives) == 0:
            gtk.gdk.beep()
            return
        compsel = CompletionSelector(alternatives)
        comp = compsel.window
        x0, y0 = self.entry.window.get_origin()
        width, height = self.entry.window.get_size()
        comp.set_gravity(gtk.gdk.GRAVITY_NORTH_EAST)
        comp.set_geometry_hints(min_height=5*height, max_height=5*height,
                		min_width=width, max_width=width)
        comp.move(x0, y0 - 5*height)
        comp.set_modal(True)
        comp.show()
        comp.realize()
        while gtk.events_pending(): gtk.main_iteration()
        gtk.gdk.keyboard_grab(comp.window)
        compsel.connect("completion", self.__completion_selected)

# class_init
gobject.type_register(Entry)


if __name__ == '__main__':
    dialog = gtk.Dialog(buttons=(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
    entry = Entry()
    entry.entry.show()
    dialog.vbox.add(entry.entry)
    dialog.set_default_size(200,100)
    def get_completions(entry, prefix):
        return [s for s in
                ("Foobar", "hello", "complete_me", "complete_you")
                if s.startswith(prefix)]
    entry.connect("completions-request", get_completions)
    dialog.run()

# EOF
