import misc
import gtkhtml2
import gtk
import xml.sax.saxutils


class Display(misc.ConsoleDisplay):
    _html_prologue = "<html><body><h1>Welcome to gNum&Sigma;&chi;&rho;</h1> Ready"
    _html_epilogue = "</body></html>"
    

    def __init__(self):
        self._html_view = gtkhtml2.View()
        self._html_doc  = gtkhtml2.Document()
        self._html_view.set_document(self._html_doc)
        self._html_view.show()

        self._vpaned = gtk.VPaned()
        self._vpaned.show()
        self._hpaned = gtk.HPaned()
        self._hpaned.show()

        sw = gtk.ScrolledWindow()
        sw.set_property("hscrollbar-policy", gtk.POLICY_AUTOMATIC)
        sw.set_property("vscrollbar-policy", gtk.POLICY_AUTOMATIC)
        sw.set_property("border-width", 0)
        sw.add(self._html_view)
        sw.show()

        frame = gtk.Frame()
        frame.set_shadow_type(gtk.SHADOW_IN)
        frame.show()
        frame.add(sw)
        self.frame = frame

        self.html_chunks = [ ]
        self.update_html()

    def get_widget(self):
        return self.frame

    def update_html(self):
        self._html_doc.open_stream("text/html")
        self._html_doc.write_stream(self._html_prologue)
        for chunk, n in zip(self.html_chunks, xrange(len(self.html_chunks))):
            self._html_doc.write_stream(chunk)
        self._html_doc.write_stream(self._html_epilogue)
        self._html_doc.close_stream()
        # TODO: try to eliminate the flicker
        adj = self._html_view.get_parent().get_vadjustment()
        adj.set_value(adj.upper)


    def display_qa(self, question, answer):
        question = question.decode("utf-8").encode("latin1")
        question = xml.sax.saxutils.escape(question)

        answer = answer.decode("utf-8").encode("latin1")
        answer = xml.sax.saxutils.escape(answer)

        i = len(self.html_chunks)

        html = r'''
<table width="100%%">
  <tr>
    <td width="10%%" align="right">
      <b>In[%(index)i]</b>
    </td>
    <td bgcolor="#e0e0e0">
      <tt>%(question)s</tt>
    </td>
  </tr>
  <tr>
    <td width="10%%" align="right">
      <b>Out[%(index)i]</b>
    </td>
    <td bgcolor="#ffffe0">
      <tt>%(answer)s</tt>
    </td>
  </tr>
</table>
''' % {
            'index': len(self.html_chunks),
            'question': question,
            'answer': answer,
            }
        #print html
        self.html_chunks.append(html)
        self.update_html()

    def display_help(self, title, body):
        body = body.decode("utf-8").encode("latin1")
        chunk = '<h1>%(title)s</h1>\n%(body)s' % vars()
        #print chunk
        self.html_chunks.append(chunk)
        self.update_html()
        
