#!/usr/bin/env python
import pygtk; pygtk.require("2.0")
import gobject
import gtk, pango
import gconf
from gettext import gettext as _

# ListStore columns
(
    COLUMN_LOADED,
    COLUMN_LOAD_ON_STARTUP,
    COLUMN_NAME,
    COLUMN_LOCATION,
    COLUMN_DESCRIPTION,
    COLUMN_LONGER_DESC,
    ) = range(6)


class ModulesWindow(object):

    def __init__(self, kernel_wrapper):
        self.kernel = kernel_wrapper.kernel
        self.kernel_wrapper = kernel_wrapper
        self.list_store = gtk.ListStore(gobject.TYPE_BOOLEAN,
                                        gobject.TYPE_BOOLEAN,
                			gobject.TYPE_STRING,
                			gobject.TYPE_STRING,
                			gobject.TYPE_STRING,
                			gobject.TYPE_STRING)
        self.create_window()

    def update_model(self):
        startup_modules = self.kernel_wrapper.get_startup_modules()
        self.list_store.clear()
        for module in self.kernel.listModules():
            iter = self.list_store.append()
            self.list_store.set(iter,
                		COLUMN_LOADED,      module.loaded,
                		COLUMN_LOAD_ON_STARTUP, module.name in startup_modules,
                		COLUMN_NAME,        module.name,
                		COLUMN_LOCATION,    module.location,
                		COLUMN_DESCRIPTION, module.description,
                		COLUMN_LONGER_DESC, module.longer_desc)
        return False

    def column_load_on_startup_toggled(self, renderer, path):
        iter = self.list_store.get_iter(path)
        name = self.list_store.get_value(iter, COLUMN_NAME)
        startup_modules = self.kernel_wrapper.get_startup_modules()
        active = not self.list_store.get_value(iter, COLUMN_LOAD_ON_STARTUP)
        self.list_store.set_value(iter, COLUMN_LOAD_ON_STARTUP, active)
        if active:
            startup_modules.append(name)
        else:
            startup_modules.remove(name)
        self.kernel_wrapper.set_startup_modules(startup_modules)

    def column_loaded_toggled(self, renderer, path):
        iter = self.list_store.get_iter(path)
        active = self.list_store.get_value(iter, COLUMN_LOADED)
        if active:
            return
        name = self.list_store.get_value(iter, COLUMN_NAME)
        self.kernel.loadModule(name)
        gobject.timeout_add(100, self.update_model)

    def create_view(self):
        self.treeview = gtk.TreeView(self.list_store)
        self.treeview.connect("cursor-changed", self.cursor_changed)

        # column for 'loaded' toggles
        renderer = gtk.CellRendererToggle()
        column = gtk.TreeViewColumn(_("Loaded"), renderer, active=COLUMN_LOADED)
        self.treeview.append_column(column)
        renderer.connect("toggled", self.column_loaded_toggled)

        # column for 'loaded' toggles
        renderer = gtk.CellRendererToggle()
        column = gtk.TreeViewColumn(_("Load on startup"), renderer, active=COLUMN_LOAD_ON_STARTUP)
        self.treeview.append_column(column)
        renderer.connect("toggled", self.column_load_on_startup_toggled)

        # column for 'name' text
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn(_("Identifier"), renderer, text=COLUMN_NAME)
        self.treeview.append_column(column)

        # column for 'description' text
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn(_("Name"), renderer, text=COLUMN_DESCRIPTION)
        self.treeview.append_column(column)

    def cursor_changed(self, treeview):
        model, iter = treeview.get_selection().get_selected()
        if iter is None:
            self.location_label.set_text("")
            self.description_label.set_text("")
        else:
            self.location_label.set_text(
                model.get_value(iter, COLUMN_LOCATION))
            self.description_label.set_text(
                model.get_value(iter, COLUMN_LONGER_DESC))

    def create_window(self):
        self.window = gtk.Dialog(title=_("Modules"),
                		 flags=gtk.DIALOG_DESTROY_WITH_PARENT,
                		 buttons=(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
        self.update_model()
        self.create_view()

        sw = gtk.ScrolledWindow()
        sw.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC);
        sw.add(self.treeview)

        paned = gobject.new(gtk.VPaned, visible=True)
        paned.pack1(sw, True, False)

        table = gtk.Table(2, 2, False)
        paned.pack2(table, False, False)
        table.set_row_spacings(8)
    
        label = gobject.new(gtk.Label, wrap=False, justify=gtk.JUSTIFY_LEFT,
                	    selectable=False, xalign=0.0, yalign=0.0)
        label.set_markup(_("<b>Location:</b>"))
        table.attach(label, 0, 1, 1, 2, gtk.FILL, gtk.FILL, 4, 4)

        label = gobject.new(gtk.Label, wrap=False, justify=gtk.JUSTIFY_LEFT,
                	    selectable=False, xalign=0.0, yalign=0.0)
        label.set_markup(_("<b>Description:</b>"))
        table.attach(label, 0, 1, 0, 1, gtk.FILL, gtk.FILL, 4, 4)

        self.location_label = gobject.new(gtk.Label, wrap=False,
                			  justify=gtk.JUSTIFY_LEFT,
                			  selectable=True,
                			  xalign=0.0, yalign=0.0)
        table.attach(self.location_label, 1, 2, 1, 2,
                     gtk.FILL, gtk.FILL, 4, 4)
        self.location_label.modify_font(pango.FontDescription("Monospace"))

        self.description_label = gobject.new(gtk.Label, wrap=True,
                			     justify=gtk.JUSTIFY_LEFT,
                			     selectable=True,
                			     xalign=0.0, yalign=0.0)
        table.attach(self.description_label, 1, 2, 0, 1,
                     gtk.FILL, gtk.FILL, 4, 4)

        self.window.vbox.add(paned)
        self.window.set_default_size(512, 384)
        self.window.show_all()


if __name__ == '__main__':
    from kernel import *
    kernel = get_kernel()
    modwin = ModulesWindow(kernel)
    modwin.window.run()
    kernel.unref()

