#! /usr/bin/env python
import pygtk
pygtk.require("2.0")

import gtk
import gobject
from gettext import gettext as _
import Numexp


_DEFINITION_MARKUP_VARIABLE  = "<span style=\"oblique\">%s</span>"
_DEFINITION_MARKUP_FUNCTION  = "<span>%s</span>"
_DEFINITION_MARKUP_NAMESPACE = "<span weight=\"bold\" underline=\"single\">%s</span>"



class Model(gtk.TreeStore):
    DEFINITION_NAME_COLUMN = 0
    DEFINITION_PATH_COLUMN = 1
    DEFINITION_TYPE_COLUMN = 2
    NUM_COLUMNS = 3

    def __init__(self):
        gtk.TreeStore.__init__(self, gobject.TYPE_STRING, # name
                	       gobject.TYPE_STRING, # path
                	       gobject.TYPE_INT  # type
                	       )
        self.update_lock = False
        self.update_list = []
        
    def __update_namespace_timeout(self, kernel):
        changes = True;
        queue = list(self.update_list)
        while changes:
            changes = False
            for ns in queue:
                for ns1 in queue:
                    if ns is ns1: continue
                    if ns == ns1 or ns1.startswith(ns):
                	# remove ns1
                	queue.remove(ns1)
                	changes = True
                	break
                if changes: break
        self.update_list = queue
        self.update_lock = True

        while self.update_list:
            queue = self.update_list
            self.update_list = []
            for ns in queue:
                self.do_update_namespace(ns, kernel)
        self.update_lock = False
        return False

    def do_update_namespace(self, ns, kernel):
        # Find namespace iterator
        nsiter = self.get_iter_root()
        not_empty = (nsiter is not None)
        is_root = (ns == "::")

        if is_root and not_empty:
            self.clear()
            not_empty = False;

        if not is_root and not_empty:
            exists, nsiter = self.__find_nsiter(ns)
            if not exists:
                # Namespace does not exist yet; create it!
                nsiter = self.append()
                # FIXME: need to extract last component of path to pass in as 'name'
                self.set(nsiter, DEFINITION_NAME_COLUMN, ns,
                	 DEFINITION_PATH_COLUMN, ns)

        # Clear all items beneath nsiter
        if not_empty:
            while 1:
                if is_root:
                    parent = None
                else:
                    parent = nsiter
                child = self.iter_children(parent)
                if child:
                    ## When modifying the model, nsiter may become invalid
                    ## save it to a path and restore it later
                    path_save = self.get_path(nsiter)
                    self.remove(child)
                    nsiter = self.get_iter(path_save)
                else:
                    break
        # Insert new fresh entries, directly from the kernel ;)
        if not_empty:
            parent = self.get_iter_root()
        else:
            parent = None
        self.__insert_namespace_contents(ns, kernel, parent)


    def update_namespace(self, ns, kernel):
         # Due to CORBA's reentrant nature, we need to ensure only one
         # namespace is being updated at any time. Other update requests
         # need to be queued for later processing.

         self.update_list.append(ns)

         if self.update_lock:
             return
         self.update_lock = True
         self.update_timeout = gobject.timeout_add\
                	       (300, self.__update_namespace_timeout, kernel)


    def __insert_namespace_contents(self, ns, kernel, nsiter):
        defs  = kernel.namespaceListing(ns)
        for def_ in defs:
            name = def_.name
            if def_.type == Numexp.DEFINITION_NAMESPACE:
                path = "%s%s::" % (ns, name)
            else:
                path = "%s%s" % (ns, name)
            iter = self.append(nsiter)

            if def_.type == Numexp.DEFINITION_VARIABLE:
                markup = _DEFINITION_MARKUP_VARIABLE
            elif def_.type == Numexp.DEFINITION_USER_FUNCTION or \
                 def_.type == Numexp.DEFINITION_BUILT_IN_FUNCTION:
                markup = _DEFINITION_MARKUP_FUNCTION
            elif def_.type == Numexp.DEFINITION_NAMESPACE:
                markup = _DEFINITION_MARKUP_NAMESPACE

            markup = markup % name
            self.set(iter, self.DEFINITION_NAME_COLUMN, markup,
                     self.DEFINITION_PATH_COLUMN, path,
                     self.DEFINITION_TYPE_COLUMN, def_.type)
            if def_.type == Numexp.DEFINITION_NAMESPACE:
                self.__insert_namespace_contents(path, kernel, iter)

    def __find_nsiter_foreachfunc(self, model, path1, iter, data):
        path = model.get_value(iter, model.DEFINITION_PATH_COLUMN)
        if path and data.path == path:
            data.found = True
            data.iter = iter
            return True
        return False

    def __find_nsiter(self, ns):
        class FindNsiterData:
            found = False
            iter = None
        data = FindNsiterData()
        data.path = ns
        self.foreach(self.__find_nsiter_foreachfunc, data)
        if not data.found:
            print "Could not find nsiter for '%s'" % ns
        return data.found, data.iter

    def get_namespace_listing(self, ns):
        if ns == '::':
            iter = self.get_iter_root()
        else:
            found, parent = self.__find_nsiter(ns)
            if not found:
                raise ValueError("Could not find namespace '%s'" % ns)
            iter = self.iter_children(parent)
        listing = []
        while iter is not None:
            listing.append(self.get_value(iter, self.DEFINITION_PATH_COLUMN))
            iter = self.iter_next(iter)
        return listing
    
class View(object):
    def __init__(self, model, console):
        assert isinstance(model, Model)
        self.console = console
        self.model = model

        sw = gtk.ScrolledWindow(); sw.show()
        sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

        sorted = gtk.TreeModelSort(model)
        self.treeview = gtk.TreeView(sorted)
        self.treeview.show()
        self.__add_columns()
        
        #self.treeview.set_rules_hint(True)
        self.treeview.get_selection().set_mode(gtk.SELECTION_SINGLE)
        sw.add(self.treeview)

        self.treeview.connect("row_activated",             self.__row_activated)
        self.treeview.connect("button_press_event", self.__button_press_event)
        self.treeview.connect("key_press_event", self.__key_press_event)

        sorted.set_sort_column_id(self.model.DEFINITION_PATH_COLUMN,
                		  gtk.SORT_ASCENDING)
        self.widget = sw

    def __add_columns(self):
        renderer = gtk.CellRendererText()
        renderer.set_property("xalign", 0.0)
        column = self.treeview.insert_column_with_attributes(
            -1, _("Definitions"), renderer,
            markup=self.model.DEFINITION_NAME_COLUMN)
        column.set_sort_indicator(True)
        column.set_sort_column_id(self.model.DEFINITION_PATH_COLUMN)


    def set_cwd(self, cwd):
        pass ## TODO

    def __row_activated(self, tree, path, column):
        model = tree.get_model()
        iter = model.get_iter(path)
        if iter is None: return
        text     = model.get_value(iter, self.model.DEFINITION_PATH_COLUMN)
        def_type = model.get_value(iter, self.model.DEFINITION_TYPE_COLUMN)
        #print "text: '%s'; type: %i" % (text, def_type)
        self.console.entry.insert_text_at_cursor(text)
        self.console.entry.grab_focus()

    def __create_popup_menu(self, def_name):
        # create popup menu
        menu = gtk.Menu()
        menu.def_name = def_name

        # Title menu item
        #item = gobject.new(gtk.MenuItem, visible=True, sensitive=False)
        #item.add(gobject.new(gtk.Label, label=def_name, visible=True))
        #menu.append(item)
        # Separator
        #menu.append(gobject.new(gtk.MenuItem, visible=True))

        # "Help" menu item
        item = gobject.new(gtk.MenuItem, visible=True,
                	   child=gobject.new(gtk.Label,
                			     label=_("_Help"),
                			     xalign=0,
                			     use_underline=True,
                			     visible=True))
        item.connect("activate", self.__popup_help_activate)
        menu.append(item)

        # "About" menu item
#         item = gobject.new(gtk.MenuItem, visible=True,
#                 	   child=gobject.new(gtk.Label,
#                 			     label=_("_About"),
#                 			     xalign=0,
#                 			     use_underline=True,
#                 			     visible=True))
#         item.connect("activate", self.__popup_about_activate)
#         menu.append(item)
        return menu

    def __key_press_event(self, tree, event):

        if (event.state == gtk.gdk.SHIFT_MASK and
            event.keyval == gtk.keysyms.F10) or \
            (event.state == 0 and
             event.keyval == gtk.keysyms.Menu):

            model, iter = tree.get_selection().get_selected()
            def_name = model.get_value(iter, self.model.DEFINITION_PATH_COLUMN)
            path = model.get_path(iter)
            column = tree.get_column(self.model.DEFINITION_NAME_COLUMN)
            rect = tree.get_cell_area(path, column)
            x0, y0 = tree.window.get_origin()
            xc = x0 + int(rect.x)
            yc = y0 + int(rect.y)
            menu = self.__create_popup_menu(def_name)
            def position_func(m):
                return xc, yc, True
            menu.popup(None, None, position_func, 0, 0)
            
    def __button_press_event(self, tree, event):
        if event.button != 3:
            return
        model = tree.get_model()
        path, x, x, x = tree.get_path_at_pos(int(event.x), int(event.y))
        iter = model.get_iter(path)
        if iter is None:
            return
        def_name = model.get_value(iter, self.model.DEFINITION_PATH_COLUMN)
        menu = self.__create_popup_menu(def_name)
        menu.popup(None, None, None, event.button, event.time)

    def __popup_help_activate(self, item):
        self.console.display_help_on(item.parent.def_name)

    def __popup_about_activate(self, item):
        print "about(%s)" % item.parent.def_name
        raise Exception('Not Implemented')


def test():
    window = gtk.Window()
    window.connect("destroy", lambda w: gtk.main_quit())
    window.set_default_size(gtk.gdk.screen_width()*2/3,
                	    gtk.gdk.screen_height()*2/3)
    model = Model()
    model.update_namespace("::", kernel.get_kernel())
    view = View(model, console=None)
    window.add(view.widget)
    window.show_all()
    gtk.main()


if __name__ == '__main__':
    test()
