import gobject
import gnome.ui
import bonobo
import gconf
import gnomevfs
import sys
import os, os.path
import gettext
import locale
import gtk
import gtk.glade
import gnumexp_conf

import kernel
import gtkexcepthook
import util
import libxml2
import traceback
from cStringIO import StringIO

from gettext import gettext as _

from console.console import Console
from xygraph.view import XYGraphView

# ========================================================

APPNAME             = "GNumExp"
GETTEXT_PACKAGE     = 'gnumexp'
VERSION             = gnumexp_conf.VERSION
GNOMELOCALEDIR      = gnumexp_conf.GNOMELOCALEDIR
UICONFIG_DIR        = "/apps/gnumexp/UIConfig"
SHOW_STATUS_BAR_KEY = UICONFIG_DIR + "/show_status_bar"
SHOW_TOOL_BAR_KEY   = UICONFIG_DIR + "/show_tool_bar"

# ========================================================
shell_ui_info = \
"""<ui>
  <menubar name='MenuBar'>
    <menu action='FileMenu'>
      <menuitem action='FileNewConsole'/>
      <menuitem action='FileNewXYGraph'/>
      <menuitem action='FileOpen'/>
      <menuitem action='FileSave'/>
      <menuitem action='FileSaveAs'/>
      <placeholder name='FileAdditions'/>
      <separator/>
      <menuitem action='FileQuit'/>
    </menu>
    <menu action='EditMenu'/>
    <menu action='ViewMenu'>
        <menuitem action='ViewToolbar'/>
        <menuitem action='ViewStatusbar'/>
    </menu>
    <menu action='HelpMenu'>
      <placeholder name='TopicListAdditions'/>
      <menuitem action='HelpBugReport'/>
      <menuitem action='HelpAbout'/>
      <menuitem action='HelpAboutNumexp'/>
    </menu>
  </menubar>
  <toolbar  name='ToolBar'>
    <!-- <toolitem action='FileNewConsole'/>
    <toolitem action='FileNewXYGraph'/>
    <toolitem action='FileQuit'/> -->
  </toolbar>
</ui>"""


class MainWindow(gtk.Window):
    toplevels = [] # class static member

    document_uri = None # URI of the currently loaded document
    persist      = None # the Bonobo::PersistFile of the control, if any.

    def __init__(self, component, parent_window=None):
        gtk.Window.__init__(self, gtk.WINDOW_TOPLEVEL)
        self.toplevels.append(self)
        self.connect("delete-event", self._delete_event)
        self.connect("destroy", self._window_destroyed)
        self.ui_manager = gtk.UIManager()
        self.ui_manager.insert_action_group(self.__create_action_group(), 0)
        self.add_accel_group(self.ui_manager.get_accel_group())
        self.kernel = kernel.NumexpKernel()

        mergeid = self.ui_manager.add_ui_from_string(shell_ui_info)
        bar = self.ui_manager.get_widget("/MenuBar")
        bar.show()

        table = gtk.Table(1, 4, False)
        self.add(table)
        table.show()

        table.attach(bar,
            # X direction #          # Y direction
            0, 1,                      0, 1,
            gtk.EXPAND | gtk.FILL,     0,
            0,                         0);

        bar = self.ui_manager.get_widget("/ToolBar")
        bar.set_tooltips(True)
        bar.show()
        table.attach(bar,
            # X direction #       # Y direction
            0, 1,                   1, 2,
            gtk.EXPAND | gtk.FILL,  0,
            0,                      0)



        # Create statusbar
        self.statusbar = gtk.Statusbar()
        table.attach(self.statusbar,
            # X direction           Y direction
            0, 1,                   3, 4,
            gtk.EXPAND | gtk.FILL,  0,
            0,                      0)
        self.statusbar.show()

        self.set_default_size(gtk.gdk.screen_width()*2/3,
                		gtk.gdk.screen_height()*2/3)
        self._read_initial_settings()
        #self.update_window_title()

        # Create document
        self.component = component(self)
        contents = self.component.get_widget()
        contents.show()
        table.attach(contents,
            # X direction           Y direction
            0, 1,                   2, 3,
            gtk.EXPAND | gtk.FILL,  gtk.EXPAND | gtk.FILL,
            0,                      0)
        self.show()

    def __create_action_group(self):
        # GtkActionEntry
        entries = (
          ( "FileMenu", None, "_File" ),               # name, stock id, label
          ( "ViewMenu", None, "_View" ),                # name, stock id, label
          ( "EditMenu", None, "_Edit" ),                # name, stock id, label
          ( "HelpMenu", None, "_Help" ),               # name, stock id, label

          ( "FileNewConsole", gtk.STOCK_NEW,           # name, stock id
            "_New Console", "<control>N",              # label, accelerator
            "Create a new console",                    # tooltip
            self.on_menu_file_new_console),

          ( "FileNewXYGraph", gtk.STOCK_NEW,           # name, stock id
            "New _Plot", "<control>N",                 # label, accelerator
            "Create a new plot",                       # tooltip
            self.on_menu_file_new_xygraph),

          ( "FileOpen", gtk.STOCK_OPEN,                # name, stock id
            "_Open","<control>O",                      # label, accelerator
            "Open a file",                             # tooltip
            self.on_menu_file_open ),
          
          ( "FileSave", gtk.STOCK_SAVE,                # name, stock id
            "_Save","<control>S",                      # label, accelerator
            "Save current file",                       # tooltip
            self.on_menu_file_save ),

          ( "FileSaveAs", gtk.STOCK_SAVE,              # name, stock id
            "Save _As...", None,                       # label, accelerator
            "Save to a file",                          # tooltip
            self.on_menu_file_save_as ),

          ( "FileQuit", gtk.STOCK_QUIT,                # name, stock id
            "_Quit", "<control>Q",                     # label, accelerator
            "Quit",                                    # tooltip
            self.on_menu_file_exit ),

          ( "HelpBugReport", None,                     # name, stock id
            "_Report a Bug", None,                     # label, accelerator
            "Report a Bug",                            # tooltip
            self.on_menu_help_bug_report ),
          
          ( "HelpAbout", gtk.STOCK_ABOUT,              # name, stock id
            "_About", None,                            # label, accelerator
            "About",                                   # tooltip
            self.on_menu_help_about ),
          
          ( "HelpAboutNumexp", gtk.STOCK_ABOUT,        # name, stock id
            "_About NumExp", None,                     # label, accelerator
            "About",                                   # tooltip
            self.on_menu_help_about_numexp ),
        );

        # GtkToggleActionEntry
        toggle_entries = (
          ( "ViewToolbar", None,                    # name, stock id
             "_Tool bar", "<control>T",             # label, accelerator
            "Toggle display of the toolbar",        # tooltip
            self.on_toggle_view_toolbar,
            True ),                                 # is_active
          ( "ViewStatusbar", None,                  # name, stock id
             "_Status bar", "<control>T",           # label, accelerator
            "Toggle display of the status bar",     # tooltip
            self.on_toggle_view_statusbar,
            True ),                                 # is_active
        )

        # Create the menubar and toolbar
        action_group = gtk.ActionGroup("AppWindowActions")
        action_group.add_actions(entries)
        action_group.add_toggle_actions(toggle_entries)

        return action_group

    def _window_destroyed(self, window):
        # remove self from global list of main windows
        self.toplevels.remove(self)
        if not self.toplevels:
            bonobo.main_quit()
        
    def _delete_event(self, window, event):
        self.close()
        return True

    def close(self, quitting=False):
        close = self.document_save_hook(can_cancel=(not quitting))
        if close:
            self.destroy()

    def on_menu_file_close(self, action):
        self.close()

    def on_menu_file_exit(self, action):
        for mw in tuple(self.toplevels):
            mw.close(True)

    def on_menu_file_new_console(self, action):
        MainWindow(Console, self)

    def on_menu_file_new_xygraph(self, action):
        MainWindow(XYGraphView, self)

    def on_menu_help_bug_report(self, action):
        gnome.ui.url_show_on_screen("https://launchpad.net/products/gnumexp/+filebug",
                                    self.get_screen())

    def on_menu_help_about(self, action):
        dialog = gtk.AboutDialog()

        dialog.set_name("gNumExp")
        dialog.set_version(gnumexp_conf.VERSION)

        dialog.set_copyright("\302\251 2001-2005 The NumExp Team")
        dialog.set_website("http://numexp.org/")

        logo = gtk.gdk.pixbuf_new_from_file(
            os.path.join(gnumexp_conf.PIXMAPSDIR, "numexp_logo.png"))
        dialog.set_logo(logo)

        dialog.set_authors([
            "Gustavo Carneiro  <gjc@inescporto.pt>",
            "Alberto Sim\303\265es  <ambs@di.uminho.pt>",
            ])
        dialog.set_artists([
            "Daniel Ketel",
            "Gustavo Carneiro  <gjc@inescporto.pt>"])
        dialog.set_documenters([
            "Alberto Sim\303\265es  <ambs@di.uminho.pt>",
            ])
        dialog.set_comments("A GUI frontend to NumExp.")
        dialog.set_transient_for(self)
        ## Close dialog on user response
        dialog.connect("response", lambda d, r: d.destroy())
        dialog.show()

    def on_menu_help_about_numexp(self, action):
        version = self.kernel.kernel.version()

        dialog = gtk.AboutDialog()

        dialog.set_name("NumExp")
        dialog.set_version(version)

        dialog.set_copyright("\302\251 1998-2005 The NumExp Team")
        dialog.set_website("http://numexp.org/")

        logo = gtk.gdk.pixbuf_new_from_file(
            os.path.join(gnumexp_conf.PIXMAPSDIR, "numexp_logo.png"))
        dialog.set_logo(logo)

        dialog.set_authors([
            "Alberto Sim\303\265es  <ambs@di.uminho.pt>",
            "Gustavo Carneiro  <gjc@inescporto.pt>",
            "David Boucher  <bouda1@wanadoo.fr>",
            ])
        dialog.set_documenters([
            "Alberto Sim\303\265es  <ambs@di.uminho.pt>",
            ])
        dialog.set_comments("A math/algebra engine and programming language.")
        dialog.set_transient_for(self)
        ## Close dialog on user response
        dialog.connect("response", lambda d, r: d.destroy())
        dialog.show()

    def on_menu_help_contents(self, action):
        gnome.help_display("gnumexp")

    def _read_initial_settings(self):
        action = self.ui_manager.get_action("/MenuBar/ViewMenu/ViewToolbar")
        action.set_active(gconf.client_get_default().get_bool(SHOW_TOOL_BAR_KEY))
        self.on_toggle_view_toolbar(action)

        action = self.ui_manager.get_action("/MenuBar/ViewMenu/ViewStatusbar")
        action.set_active(gconf.client_get_default().get_bool(SHOW_STATUS_BAR_KEY))
        self.on_toggle_view_statusbar(action)
        
        # ...
        # FileSave and FileSaveAs are only enabled if the component
        # has the Bonobo::PersistStream interface.
        #if self.persist is None:
        #    enable_save = "0"
        #else:
        #    enable_save = "1"
        #ui_component.set_prop("/commands/FileSave", "sensitive", enable_save)
        #ui_component.set_prop("/commands/FileSaveAs", "sensitive", enable_save)


    def on_toggle_view_toolbar(self, action):
        gconf.client_get_default().set_bool(SHOW_TOOL_BAR_KEY, action.get_active())
        self.ui_manager.get_widget("/ToolBar").props.visible = action.get_active()

    def on_toggle_view_statusbar(self, action):
        gconf.client_get_default().set_bool(SHOW_STATUS_BAR_KEY, action.get_active())
        self.statusbar.props.visible = action.get_active()

    def save_to(self, uri):
        try:
            content = StringIO()
            self.component.save_to(content)
            handle = gnomevfs.create(uri, gnomevfs.OPEN_WRITE)
            handle.write(content.getvalue())
            handle.close()
            save_ok = True
        except gnomevfs.Error, ex:
            save_ok = False
            bonobo_error = str(ex.__class__)
            dialog = gtk.MessageDialog(parent  = self,
                		       flags   = gtk.DIALOG_DESTROY_WITH_PARENT,
                		       type    = gtk.MESSAGE_ERROR,
                		       buttons = gtk.BUTTONS_OK,
                		       message_format = _("Error saving file"))
            dialog.format_secondary_markup(_("Error '<tt>%s</tt>' occurred"
                                             " while trying to save file '<tt>%s</tt>'")
                                           % (gobject.markup_escape_text(bonobo_error),
                                              gobject.markup_escape_text(gnomevfs.format_uri_for_display(uri))))
            dialog.run()
            dialog.destroy()
        if save_ok:
            self.document_uri = uri
            self.update_window_title()
        return save_ok

    def _component_invoke_load_from(self, uri):
        try:
            self.component.load_from_uri
        except AttributeError:
            contents = gnomevfs.read_entire_file(uri)
            io = StringIO(contents)
            self.component.load_from(io)
        else:
            self.component.load_from_uri(uri)

    def load_from(self, uri):
        try:
            self._component_invoke_load_from(uri)
        except Exception, ex:
            traceback.print_exc()
            bonobo_error = str(ex.__class__)
            dialog = gtk.MessageDialog(parent  = self,
                		       flags   = gtk.DIALOG_DESTROY_WITH_PARENT,
                		       type    = gtk.MESSAGE_ERROR,
                		       buttons = gtk.BUTTONS_OK,
                		       message_format = _("Error loading file"))
            dialog.format_secondary_markup(_("Error '<tt>%s</tt>' occurred"
                                             " while trying to load file '<tt>%s</tt>'")
                                           % (gobject.markup_escape_text(bonobo_error),
                                              gobject.markup_escape_text(gnomevfs.format_uri_for_display(uri))))
            dialog.run()
            dialog.destroy()
            self.close()
            return
        self.document_uri = uri
        self.update_window_title()

    def on_menu_file_save(self, action):
        if self.document_uri is not None:
            self.save_to(self.document_uri)
        else:
            self.on_menu_file_save_as(None)

    def on_menu_file_save_as(self, action):
        self.save_as()

    def save_as(self):
        sel = gtk.FileChooserDialog("Save...", None,
                                    gtk.FILE_CHOOSER_ACTION_SAVE,
                                    (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                     gtk.STOCK_SAVE, gtk.RESPONSE_OK))
        sel.set_default_response(gtk.RESPONSE_OK)
        sel.set_local_only(False)
        sel.set_do_overwrite_confirmation(True)
        sel.set_current_name(_("Untitled Plot.xyg"))

        filter = gtk.FileFilter()
        filter.set_name(_("gNumExp Plots"))
        filter.add_mime_type("application/x-xygraph")
        sel.add_filter(filter)

        filter = gtk.FileFilter()
        filter.set_name(_("All files"))
        filter.add_pattern("*")
        sel.add_filter(filter)

        sel.set_transient_for(self)
        resp = sel.run()
        if resp != gtk.RESPONSE_OK:
            sel.destroy()
            return False
        
        uri = sel.get_uri()
        sel.destroy()
        return self.save_to(uri)

    def on_menu_file_open(self, action):
        sel = gtk.FileChooserDialog(_("Open..."),
                                    None,
                                    gtk.FILE_CHOOSER_ACTION_OPEN,
                                    (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                     gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        sel.set_default_response(gtk.RESPONSE_OK)
        sel.set_local_only(False)

        filter = gtk.FileFilter()
        filter.set_name(_("gNumExp Plots"))
        filter.add_mime_type("application/x-xygraph")
        sel.add_filter(filter)

        filter = gtk.FileFilter()
        filter.set_name(_("NumExp Scritps"))
        filter.add_mime_type("application/x-numexp")
        sel.add_filter(filter)

        sel.set_transient_for(self)

        resp = sel.run()
        if resp != gtk.RESPONSE_OK:
            sel.destroy()
            return

        uri = sel.get_uri()
        sel.destroy()
        info = gnomevfs.get_file_info(uri, gnomevfs.FILE_INFO_GET_MIME_TYPE)
        if info.mime_type == 'application/x-xygraph':
            mainwin = MainWindow(XYGraphView)
        elif info.mime_type == 'application/x-numexp':
            if isinstance(self.component, Console):
                mainwin = self
            else:
                mainwin = MainWindow(Console)
        else:
            dialog = gtk.MessageDialog(type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK,
                                       message_format=(
                "Could not load file: unknown mime type %s" % (info.mime_type,)))
            dialog.run()
            dialog.destroy()
            return
        mainwin.load_from(uri)


    def get_document_modified(self):
        if isinstance(self.component, gobject.GObject):
            try:
                return self.component.props.modified
            except AttributeError:
                pass
        return False

    def document_save_hook(self, can_cancel):
        if not self.get_document_modified():
            return True

        if self.document_uri is None:
            msg = _("<b>Save the changes to the document before closing?</b>")
        else:
            msg = (_("<b>Save the changes to document"
                    " \"<tt>%s</tt>\" before closing?</b>") %
                   gobject.markup_escape_text(gnomevfs.format_uri_for_display(self.document_uri)))

        msgbox = gtk.MessageDialog(parent  = self,
                		   flags   = (gtk.DIALOG_MODAL|
                			      gtk.DIALOG_DESTROY_WITH_PARENT),
                		   type    = gtk.MESSAGE_QUESTION, 
                		   buttons = gtk.BUTTONS_NONE)
        msgbox.set_markup(msg)
        msgbox.format_secondary_text(_("If you don't save, "
                                       "the changes will be permanently lost."))
        msgbox.add_button(_("Close _without saving"), gtk.RESPONSE_NO)
        if can_cancel:
            msgbox.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        msgbox.add_button(gtk.STOCK_SAVE, gtk.RESPONSE_YES)
        msgbox.set_default_response(gtk.RESPONSE_YES)

        msgbox.set_resizable(False)
        msgbox.show(); msgbox.present()
        ret = msgbox.run()
        msgbox.destroy()

        if ret == gtk.RESPONSE_YES:
            if self.document_uri:
                save_ok = self.save_to(self.document_uri)
                close = save_ok
            else:
                save_ok = self.save_as()
                close = save_ok
        elif ret == gtk.RESPONSE_NO:
            close = True
        elif ret == gtk.RESPONSE_CANCEL:
            close = False
        else:
            close = False
        return close

    def update_window_title(self):
        if isinstance(self.component, Console):
            document = _("Console")
        elif isinstance(self.component, XYGraphView):
            if self.document_uri is None:
                document = _("Plot")
            else:
                document = gnomevfs.URI(self.document_uri).short_name
        else:
            assert 0, "unexpected moniker " + str(self.moniker)
        self.set_title("%s - GNumExp" % document)

def _about_dialog_link_hook(about, link, scheme):
    gnome.ui.url_show_on_screen(scheme + link, about.get_screen())


popt_table = [
       ('plot-window', 'p', None, None, 0,
        'Start a Plot window instead of a Console', ''),
       ]

def new_instance_cb(unused_app, argc, argv):
    num_toplevels = len(MainWindow.toplevels)
    leftover_args, argdict = gnome.popt_parse(argv, popt_table)
    if argdict['plot-window']:
        default_component = XYGraphView
    else:
        default_component = Console
    for arg in leftover_args:
        uri = gnomevfs.make_uri_from_shell_arg(arg)
        info = gnomevfs.get_file_info(uri, gnomevfs.FILE_INFO_GET_MIME_TYPE)
        if info.mime_type == 'application/x-xygraph':
            mainwin = MainWindow(XYGraphView)
        elif info.mime_type == 'application/x-numexp':
            mainwin = MainWindow(Console)
        else:
            dialog = gtk.MessageDialog(type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK,
                                       message_format=(
                "Could not load file: unknown mime type %s" % (info.mime_type,)))
            dialog.run()
            dialog.destroy()
            return
        mainwin.load_from(uri)
    ## If no documents were loaded, create a default main window
    if len(MainWindow.toplevels) == num_toplevels:
        MainWindow(default_component)
    return False

def register_stock_icons():
    ''' This function registers our custom toolbar icons, so they
        can be themed.
    '''
    factory = gtk.IconFactory()
    ## Add our custom icon factory to the list of defaults
    factory.add_default()

    def add_simple_stock_icon(icon_name, file_name):
        pixbuf = gtk.gdk.pixbuf_new_from_file(os.path.join(
            gnumexp_conf.PIXMAPSDIR, file_name))
        icon_set = gtk.IconSet(pixbuf)
        factory.add(icon_name, icon_set)

    add_simple_stock_icon('gnumexp-definitions', "defs_button.png")
    add_simple_stock_icon('gnumexp-grid', "grid_button.png")
    add_simple_stock_icon('gnumexp-legend', "legend_button.png")
    add_simple_stock_icon('gnumexp-axis', "axis_button.png")


def main(argv):
    # Locale and I18N stuff
    #os.environ['LC_NUMERIC'] = 'C'
    #locale.setlocale(locale.LC_NUMERIC, 'C')
    gettext.install(GETTEXT_PACKAGE, GNOMELOCALEDIR)
    gettext.bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR)
    gettext.textdomain(GETTEXT_PACKAGE)
    gtk.glade.bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR)
    try:
        gtk.glade.textdomain(GETTEXT_PACKAGE)
    except AttributeError:
        print "gtk.glade.textdomain not found; i18n may not work"
    gtk.about_dialog_set_email_hook(_about_dialog_link_hook, "mailto:")
    gtk.about_dialog_set_url_hook(_about_dialog_link_hook, "")
    icon = gtk.gdk.pixbuf_new_from_file(os.path.join(gnumexp_conf.ICONDIR, "gnumexp_icon.png"))
    gtk.window_set_default_icon_list(icon); del icon
    gnome.ui.authentication_manager_init()
    prog = gnome.program_init("gnumexp", gnumexp_conf.VERSION,
                              gnome.libgnome_module_info_get(),
                              sys.argv, popt_table,
                              properties={"app-prefix": gnumexp_conf.PREFIX,
                                          "app-sysconfdir": gnumexp_conf.SYSCONFDIR,
                                          "app-datadir": gnumexp_conf.DATADIR,
                                          "app-libdir": gnumexp_conf.LIBDIR,
                                          })
    register_stock_icons()
    libxml2.loadCatalog(gnumexp_conf.MATHML_DTD)

    new_instance_cb(None, len(argv), argv)

    bonobo.main()
    return 0

