import goocanvas
from misc import *
from math import fabs
import gtk, gtk.gdk
from gettext import gettext as _
from misc import Ticks
import pango

#class AxisLineOverlay(NxpCanvasOverlay):
#    def __init__(self):
#        NxpCanvasOverlay.__init__(self)
#        self.connect("draw", self.__class__.real_draw)
#
#    def real_draw(self, drawable, gc, t):
#        drawable.draw_rectangle(gc, 1, self.bbox.x, self.bbox.y,
#                		self.bbox.width,
#                		self.bbox.height)

(
    GTK_ANCHOR_CENTER,
    GTK_ANCHOR_NORTH,
    GTK_ANCHOR_NORTH_WEST,
    GTK_ANCHOR_NORTH_EAST,
    GTK_ANCHOR_SOUTH,
    GTK_ANCHOR_SOUTH_WEST,
    GTK_ANCHOR_SOUTH_EAST,
    GTK_ANCHOR_WEST,
    GTK_ANCHOR_EAST,
    ) = range(9)



class Axis(goocanvas.Group):

    __gproperties__ = {

        'position': (float,
                     'position in canvas coordinates',
                     'position in canvas coordinates',
                     -1e99, 1e99, 0.0,
                     gobject.PARAM_READWRITE,
                     ),

        }
    
    def __init__(self, plot_area):
        goocanvas.Group.__init__(self)
        self.plot_area = plot_area
        self.lineattr = None
        self.ticks = None # ticks in function units
        self.ticks_plunit = None # ticks in plot units
        self.axis_line = goocanvas.Path() # main axis line
        self.add_child(self.axis_line)
        self.axis_ticks = goocanvas.Path() # tick marks
        self.add_child(self.axis_ticks)
        self.axis_numbers = [] # dynamic list of goocanvas.Text()
        self.position = None # axis position in canvas coordinates
        self.math_position = None # axis position in math coordinates
        self.other_axis_position = None # other axis position in canvas coordinates

    def do_set_property(self, pspec, value):
        if pspec.name == 'position':
            self.position = value
        else:
            raise TypeError

    def do_get_property(self, pspec):
        if pspec.name == 'position':
            return self.position
        else:
            raise TypeError

    def draw(self):
        '''must be subclassed, only common functionality here'''
        self.lineattr.apply_to_goo_canvas_item(self.axis_line, self.plot_area.width_scale)
        self.lineattr.apply_to_goo_canvas_item(self.axis_ticks, self.plot_area.width_scale)
        for number in self.axis_numbers:
            number.props.visibility = goocanvas.ITEM_INVISIBLE
            number.props.font_desc = pango.FontDescription("Sans 12")

    def _get_number_item(self, number_index):
        try:
            text_item = self.axis_numbers[number_index]
        except IndexError:
            text_item = goocanvas.Text()
            self.axis_numbers.append(text_item)
            self.add_child(text_item)
        text_item.props.visibility = goocanvas.ITEM_VISIBLE
        return text_item
        
    def popup_menu(self, event):
        menu = gtk.Menu()

        item = gtk.ImageMenuItem(stock_id=gtk.STOCK_PROPERTIES)
        item.show()
        item.connect("activate", self.display_properties_dialog)
        menu.append(item)

        menu.popup(parent_menu_shell=None, parent_menu_item=None,
                   func=None, button=event.button,
                   activate_time=event.time)
        menu.connect("unmap", lambda m: m.destroy())
        
    def display_properties_dialog(self, *args):
        pass

    # << XML load/save >>
    def to_xml(self, doc):
        elem = doc.createElement(self.axis_xml_name)
        elem.setAttribute("position", str(self.math_position))
        return elem

    def from_xml(self, elem):
        assert isinstance(elem, dom.Element)
        assert elem.tagName == self.axis_xml_name
        self.math_position = float(elem.getAttribute("position"))


class HorizAxis(Axis):
    axis_xml_name = "haxis"
    
    def __init__(self, plot_area):
        Axis.__init__(self, plot_area)

    def update_derived_parameters(self):
        # dy is the half the length of the tick marks
        self.dy       = self.attr.get_linewidth(self.width_scale)*3
        self.fontsize = self.width_scale*30
        self.fontname = "Sans Regular"

    def draw(self):
        if self.ticks_plunit is None or self.position is None:
            return
        Axis.draw(self)
        self.axis_line.props.data = "M %f %f L %f %f" % (
            self.plot_area.bbox.x1, self.position + 0.5,
            self.plot_area.bbox.x2, self.position + 0.5)
        start  = self.ticks_plunit.start + 0.5
        div    = self.ticks_plunit.div
        if not div > 0:
            return
        subdiv = self.ticks_plunit.subdiv
        limit  = self.ticks_plunit.limit + 0.5
        y0 = self.position + 0.5
        dy = self.axis_line.props.line_width*3
        y1, y2 = y0 - dy, y0 + dy
        
        x = start - div
        xr = self.ticks.start - self.ticks.div
        data = []

        number_index = 0
        while x < limit:
            if x >= start and (self.other_axis_position is None
                               or fabs(x - self.other_axis_position) >= div/2):
                data.append("M %f %f L %f %f" % (x, y1, x, y2))
                ## draw numbers
                text_item = self._get_number_item(number_index)
                if math.fabs(xr/self.ticks.div) < 1e-6:
                    text_item.props.text = "0"
                else:
                    text_item.props.text = "%G" % (xr,)
                text_item.props.x = x
                text_item.props.y = y2
                text_item.props.anchor = GTK_ANCHOR_NORTH
            x  += div
            xr += self.ticks.div
            number_index += 1
        self.axis_ticks.props.data = '\n'.join(data)

    def display_properties_dialog(self, *args):
        self.plot_area.view.create_haxis_properties_dialog()



# class AxisLine(CanvasObjectBase):
#     def __init__(self, attr, width_scale, pos):
#         """
#         @attr: LineAttr
#         @width_scale: float
#         @pos: (float) position (either X or Y) in plot coordinates
#         """
#         super(AxisLine, self).__init__()
#         self.set_values(attr, width_scale, pos)
#         self.set_property('interaction-mask', 0)
#         self.set_property('whole-area-interaction', 0)
#         self.overlay = AxisLineOverlay()
#         self.overlay.set_bbox(0, 0, 0, 0)
#         self.connect("notify::canvas", self.__notify_canvas)
#         self.cursor = None

#     def real_size_request(self):
#         return NxplotPoint(self.linewidth, self.linewidth)

#     def real_enter(self):
#         self.canvas.set_cursor(self.cursor)

#     def real_leave(self):
#         self.canvas.set_cursor(None)

#     def __notify_canvas(self, *args):
#         self.overlay.set_canvas(self.canvas)

#     def set_values(self, attr, width_scale, pos):
#         self.attr        = attr
#         self.width_scale = width_scale
#         self.pos         = None # position in canvas coordinates
#         self.linewidth   = self.attr.get_linewidth(self.width_scale)
#         self.queue_draw()

#     def set_pos(self, pos):
#         self.pos = pos
#         self.queue_draw()

#     def real_button_press(self, event):
#         if self.drag_state: return 1
#         if event.button == 3:
#             self.container.popup_menu(event)
#             return
#         if event.button != 1: return 0
#         self.drag_state = 1
#         self.overlay_geometry_save = self.overlay_geometry
#         self.drag_origin = event.x, event.y
#         self.overlay.hide()
#         self.overlay.set_bbox(*map(int, self.overlay_geometry))
#         self.overlay.show()
#         self.container.pointer_grab(self, 1)
#         return 1


# class HorizAxisLine(AxisLine):

#     def __init__(self, *args):
#         AxisLine.__init__(self, *args)
#         self.drag_state = 0
#         self.cursor = gtk.gdk.Cursor(gtk.gdk.SB_V_DOUBLE_ARROW)

#     def real_configure(self, bbox, interaction):
#         if interaction != Interaction.MOVE_RESIZE:
#             return
#         pl = self.get_plotter()
#         if isinstance(pl, NxplotGP):
#             return
#         self.y  = (bbox.y1 + bbox.y2) / 2
#         self.x1 = bbox.x1
#         self.x2 = bbox.x2
#         x1, y1 = pl.space2window(bbox.x1, bbox.y1)
#         x2, y2 = pl.space2window(bbox.x2, bbox.y2)
#         self.overlay_geometry = (x1, y2, 0.5 + x2 - x1, 0.5 + y1 - y2)
#         #print self.overlay_geometry
#         #self.overlay_geometry = (self.wbbox.x, self.wbbox.y,
#         #        		 self.wbbox.width, self.wbbox.height)

#     def real_draw(self, pl):
#         self.attr.apply_to(pl, self.width_scale)
#         pl.endpath()
#         pl.line(self.x1, self.y, self.x2, self.y)
#         pl.endpath()


#     def real_motion_notify(self, event):
#         if not self.drag_state: return 0
#         self.overlay.hide()
#         delta = (event.y - self.drag_origin[1])
#         self.overlay_geometry = (self.overlay_geometry_save[0],
#                 		 self.overlay_geometry_save[1] + delta,
#                 		 self.overlay_geometry_save[2],
#                 		 self.overlay_geometry_save[3])
#         self.overlay.set_bbox(*map(int, self.overlay_geometry))
#         self.overlay.show()


#     def real_button_release(self, event):
#         if not self.drag_state: return 1
#         self.drag_state = 0
#         self.overlay.hide()
#         self.overlay_geometry = self.overlay_geometry_save
#         self.container.pointer_ungrab()
#         pl = self.get_plotter()
#         x0, y0 = pl.window2space(*self.drag_origin)
#         x1, y1 = pl.window2space(event.x, event.y)
#         delta = y1 - y0
#         self.set_pos(self.pos + delta)
#         self.container.line_position_changed(self.pos)




# class VertAxisLine(AxisLine):

#     def __init__(self, *args):
#         AxisLine.__init__(self, *args)
#         self.drag_state = 0
#         self.cursor = gtk.gdk.Cursor(gtk.gdk.SB_H_DOUBLE_ARROW)


#     def real_configure(self, bbox, interaction):
#         if interaction != Interaction.MOVE_RESIZE:
#             return
#         pl = self.get_plotter()
#         if isinstance(pl, NxplotGP):
#             return
#         self.x  = (bbox.x1 + bbox.x2) / 2
#         self.y1 = bbox.y1
#         self.y2 = bbox.y2
#         x1, y1 = pl.space2window(bbox.x1, bbox.y1)
#         x2, y2 = pl.space2window(bbox.x2, bbox.y2)
#         self.overlay_geometry = (x1, y2, 0.5 + x2 - x1, 0.5 + y1 - y2)
#         #print self.overlay_geometry
#         #self.overlay_geometry = (self.wbbox.x, self.wbbox.y,
#         #        		 self.wbbox.width, self.wbbox.height)

#     def real_draw(self, pl):
#         self.attr.apply_to(pl, self.width_scale)
#         pl.endpath()
#         pl.line(self.x, self.y1, self.x, self.y2)
#         pl.endpath()

#     def real_motion_notify(self, event):
#         if not self.drag_state: return 0
#         self.overlay.hide()
#         delta = (event.x - self.drag_origin[0])
#         self.overlay_geometry = (self.overlay_geometry_save[0] + delta,
#                 		 self.overlay_geometry_save[1],
#                 		 self.overlay_geometry_save[2],
#                 		 self.overlay_geometry_save[3])
#         self.overlay.set_bbox(*map(int, self.overlay_geometry))
#         self.overlay.show()

#     def real_button_release(self, event):
#         if not self.drag_state: return 1
#         self.drag_state = 0
#         self.overlay.hide()
#         self.overlay_geometry = self.overlay_geometry_save
#         self.container.pointer_ungrab()
#         pl = self.get_plotter()
#         x0, y0 = pl.window2space(*self.drag_origin)
#         x1, y1 = pl.window2space(event.x, event.y)
#         delta = x1 - x0
#         self.set_pos(self.pos + delta)
#         self.container.line_position_changed(self.pos)


class VertAxis(Axis):
    axis_xml_name = "vaxis"
    
    def __init__(self, plot_area):
        Axis.__init__(self, plot_area)

    def draw(self):
        if self.ticks_plunit is None or self.position is None:
            return
        Axis.draw(self)
        self.axis_line.props.data = "M %f %f L %f %f" % (
            self.position + 0.5, self.plot_area.bbox.y1,
            self.position + 0.5, self.plot_area.bbox.y2)
        start  = self.ticks_plunit.start + 0.5
        div    = self.ticks_plunit.div
        if not div < 0:
            return
        subdiv = self.ticks_plunit.subdiv
        limit  = self.ticks_plunit.limit + 0.5
        x0 = self.position + 0.5
        dx = self.axis_line.props.line_width*3
        x1, x2 = x0 - dx, x0 + dx
        
        y = start - div
        yr = self.ticks.start - self.ticks.div
        data = []

        number_index = 0
        while y >= limit:
            if y <= start and (self.other_axis_position is None
                               or fabs(y - self.other_axis_position) >= -div/2):
                data.append("M %f %f L %f %f" % (x1, y, x2, y))
                ## draw numbers
                text_item = self._get_number_item(number_index)
                if math.fabs(yr/self.ticks.div) < 1e-6:
                    text_item.props.text = "0"
                else:
                    text_item.props.text = "%G" % (yr,)
                text_item.props.x = x2
                text_item.props.y = y
                text_item.props.anchor = GTK_ANCHOR_WEST
            y  += div
            yr += self.ticks.div
            number_index += 1
        self.axis_ticks.props.data = '\n'.join(data)

    def display_properties_dialog(self, *args):
        self.plot_area.view.create_vaxis_properties_dialog()

