#! /usr/bin/env python

import gtk, gobject
import gnumexp_conf
from plotobject import *
from gettext import gettext as _

__all__ = ['PlotObjectListModel', 'PlotObjectListWindow']


COLUMN_VISIBLE    = 0
COLUMN_LABEL      = 1
COLUMN_TYPE       = 2
COLUMN_CONTENTS   = 3
COLUMN_EVERYTHING = 4


class PlotObjectListModel(gobject.GObject):

    def __init__(self):
        store = gtk.ListStore(gobject.TYPE_BOOLEAN,
                	      gobject.TYPE_STRING,
                	      gobject.TYPE_STRING,
                	      gobject.TYPE_STRING,
                	      gobject.TYPE_OBJECT)
        self.store = store
        self.selections = [] # list of gtk.TreeSelection

    def get_row_values(self, iter):
        data = self.store.get_value(iter, COLUMN_EVERYTHING)
        return (data.get_data('values'), data.get_data('user-data'))

    def set_row_values(self, iter, values, user_data):
        data = gobject.GObject()
        data.set_data('values', values)
        data.set_data('user-data', user_data)
        self.store.set(iter,
                       COLUMN_VISIBLE,    True,
                       COLUMN_LABEL,      values['label'],
                       COLUMN_TYPE,       values['type'],
                       COLUMN_CONTENTS,   str(values['contents']),
                       COLUMN_EVERYTHING, data)
        
    def _object_properties_cb(self, dialog, values):
        foo, data = self.get_row_values(dialog.iter)
        self.set_row_values(dialog.iter, values, data)
        self.row_changed(dialog.iter, values, data)

    def create_properties_dialog(self, iter):
        data = self.store.get_value(iter, COLUMN_EVERYTHING)
        values = data.get_data('values')

        ## a cross-layer violation, yeah I know... :|
        plot_object = data.get_data('user-data')
        kernel = plot_object.kernel.kernel
        cls = plot_object.dialog_class
        
        win = cls(kernel, True)
        win.iter = iter
        win.set_values(values)
        win.connect("apply", self._object_properties_cb)
        win.dialog.show()
        return win

    def remove_object(self, iter):
        values, data = self.get_row_values(iter)
        self.store.remove(iter)
        self.row_deleted(values, data)

    def add_object(self, values, user_data):
        iter = self.store.append()
        self.set_row_values(iter, values, user_data)
        #self.row_changed(iter, values)

    def add_new_row(self, values):
        iter = self.store.append()
        user_data = self.row_added(iter, values)
        self.set_row_values(iter, values, user_data)

    def row_changed(self, iter, values, data):
        """Called when a row has been changed"""
        print "** row_changed", values

    def row_added(self, iter, values):
        """Called when a row has been added. Should return the
        user_data argument to associate with the new row."""
        print "** row_added", values
        return None

    def row_deleted(self, values, data):
        pass

    def row_selected(self, values, data):
        pass

    def _new_object_cb(self, dialog, values):
        self.add_new_row(values)

    def create_new_object_dialog(self, kernel, dialog_class):
        win = dialog_class(kernel, False)
        win.dialog.show()
        win.connect("apply", self._new_object_cb)
        return win

    def find(self, user_data):
        iter = self.store.get_iter_first()
        while iter is not None:
            values, user_data1 = self.get_row_values(iter)
            if user_data1 is user_data:
                return iter
            iter = self.store.iter_next(iter)

    __selection_changed_lock = 0
    def _selection_changed(self, selection):
        if self.__selection_changed_lock:
            return
        self.__selection_changed_lock = 1
        model, iter = selection.get_selected()
        if iter is None:
            return
        for sel in self.selections:
            if sel is not selection:
                selection.select_iter(iter)
        self.__selection_changed_lock = 0
        values, data = self.get_row_values(iter)
        self.row_selected(values, data)
    
    def select_row(self, iter):
        for sel in self.selections:
            sel.select_iter(iter)
    
    def register_tree_view(self, treeview):
        selection = treeview.get_selection()
        selection.set_mode(gtk.SELECTION_SINGLE)
        self.selections.append(selection)
        selection.connect("changed", self._selection_changed)
        treeview.connect("destroy", self._unregister_tree_view)

    def _unregister_tree_view(self, treeview):
        selection = treeview.get_selection()
        self.selections = filter(lambda sel: sel is not selection,
                		 self.selections)
        

class PlotObjectListWindow:

    def __init__(self, model):
        assert isinstance(model, PlotObjectListModel)
        self.model = model
        self._create_window(model.store)
        self.model.register_tree_view(self.treeview)

    def _visible_toggled(self, cell, path, model):
        # get toggled iter
        iter = model.get_iter((int(path),))
        visible = model.get_value(iter, COLUMN_VISIBLE)

        # do something with the value
        visible = not visible
        # set new value
        model.set(iter, COLUMN_VISIBLE, visible)

    def _add_columns (self):
        treeview = self.treeview
        model = treeview.get_model()

        # column for 'visible' toggles
        #renderer = gtk.CellRendererToggle()
        #renderer.connect('toggled', self._visible_toggled, model)
        #column = gtk.TreeViewColumn('Visible?', renderer, active=COLUMN_VISIBLE)
        #treeview.append_column (column)

        # column for label
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn(_('Label'), renderer, text=COLUMN_LABEL)
        treeview.append_column(column)

        # column for description
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn(_('Type'), renderer, text=COLUMN_TYPE)
        treeview.append_column(column)

        # column for contents
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn(_('Contents'), renderer, text=COLUMN_CONTENTS)
        treeview.append_column(column)

    def show(self):
        self.win.show_all()

    def _create_window(self, model):
        win = gtk.Dialog(_('Plot object list'), None, gtk.DIALOG_DESTROY_WITH_PARENT,
                         (gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
        win.set_border_width(12)
        win.set_has_separator(False)

        hbox = gtk.HBox(False, 6)
        win.vbox.add(hbox)

        sw = gtk.ScrolledWindow()
        sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        sw.set_policy(gtk.POLICY_AUTOMATIC,
                      gtk.POLICY_AUTOMATIC)
        hbox.pack_start(sw, True, True, 0)

        treeview = gtk.TreeView(model)
        treeview.set_search_column(COLUMN_LABEL)
        self.treeview = treeview
        treeview.connect("row-activated", self._row_activated)
        
        sw.add(treeview)
        self._add_columns()
        win.set_default_size(640, 256)

        vbox = gtk.VButtonBox()
        vbox.set_layout(gtk.BUTTONBOX_START)
        vbox.set_spacing(6)
        hbox.pack_start(vbox, False, False, 0)

        # properties button
        bt = gtk.Button(stock=gtk.STOCK_PROPERTIES)
        bt.connect('clicked', self._properties_button_clicked)
        vbox.add(bt)

        # remove button
        bt = gtk.Button(stock=gtk.STOCK_DELETE)
        bt.connect('clicked', self._remove_button_clicked)
        vbox.add(bt)

        ## close window when close button is pressed
        win.connect("response", lambda w, r: w.destroy())

        self.win = win

        if __name__ == '__main__': # test mode
            win.connect('destroy', lambda win: gtk.main_quit())
            win.show_all()
            gtk.main()


    def _close_button_clicked(self, bt):
        self.win.destroy()

#     def _add_object_cb(self, wid, response, dialog):
#         if response == gtk.RESPONSE_ACCEPT:
#             self.model.add_new_row(dialog.get_values())

#     def create_new_plot_object_dialog(self):
#         win = PlotObjectDialog()
#         win.dialog.set_transient_for(self.win)
#         return win

#     def _add_button_clicked(self, bt):
#         win = self.create_new_plot_object_dialog()
#         win.dialog.show()
#         win.dialog.connect("response", self._add_object_cb, win)

    def _properties_button_clicked(self, bt):
        model, iter = self.treeview.get_selection().get_selected()
        if iter is None:
            return
        win = self.model.create_properties_dialog(iter)
        win.dialog.set_transient_for(self.win)
        win.dialog.show()

    def _remove_button_clicked(self, bt):
        model, iter = self.treeview.get_selection().get_selected()
        if iter is None: return
        self.model.remove_object(iter)


    def _row_activated(self, treeview, path, column):
        node = self.model.store.get_iter(path)
        win = self.model.create_properties_dialog(node)
        win.dialog.set_transient_for(self.win)
        win.dialog.show()


if __name__ == '__main__':
    gnumexp_conf.GLADEDIR = '.'
    model = PlotObjectListModel()
    view  = PlotObjectListWindow(model)

