import gobject
import pango
import gtk
import bonobo
import CORBA
import warnings

class ExpressionEntry(gtk.EventBox):
    __gproperties__ = { 'variables-list' :
                        (object,
                         'List of variables',
                         'List of variables found in the expression',
                         gobject.PARAM_READABLE)
                        }
    __gsignals__ = { 'expose-event' : 'override',
                     'size-request': 'override',
                     'size-allocate': 'override',
                     }

    def __init__(self, kernel):
        gtk.EventBox.__init__(self)
        gtk.widget_push_composite_child()
        self.entry = gtk.Entry()
        self.entry.modify_font(pango.FontDescription("Monospace"))
        self.entry.show()
        self.entry.set_has_frame(False)
        self.entry.set_activates_default(True)
        self.img = gtk.Image()
        icon_size = gtk.ICON_SIZE_SMALL_TOOLBAR
        self.img.set_from_stock(gtk.STOCK_DIALOG_WARNING, icon_size)
        self.img.set_sensitive(False)
        self.img.show()
        eventbox = gtk.EventBox()
        eventbox.set_visible_window(True)
        eventbox.set_above_child(False)
        eventbox.add(self.img)
        w, h = gtk.icon_size_lookup(icon_size)
        w += 4
        eventbox.set_size_request(w, h)
        eventbox.show()
        self.entry.connect("style-set", self.__entry_style_set, eventbox)
        hbox = gtk.HBox(False, 0)
        hbox.show()
        hbox.pack_start(eventbox, False, False, 0)
        hbox.add(self.entry)
        self.add(hbox)
        self.kernel = kernel
        self.__timeout = None
        self.entry.connect("changed", self.__entry_changed)
        ## We need to queue a redraw when focus changes, to comply with themes
        ## (like Clearlooks) which draw focused and unfocused entries differently.
        self.entry.connect_after("focus-in-event", self.__entry_focus_change_cb)
        self.entry.connect_after("focus-out-event", self.__entry_focus_change_cb)
        self.__varlist = []
        gtk.widget_pop_composite_child()

    def __set_varlist(self, varlist):
        if varlist != self.__varlist:
            self.__varlist = varlist
            self.notify("variables-list")

    def validate(self):
        expr = self.entry.get_text()
        if expr:
            try:
                el = self.kernel.parseString(expr)
            except CORBA.UserException:
                el = None
        else:
            el = None
        if el is None:
            self.img.set_sensitive(True)
            return False
        else:
            try:
                self.img.set_sensitive(False)
                varlist = []
                for var in self.kernel.elementListVariables(el):
                    if var not in varlist:
                        varlist.append(var)
                self.__set_varlist(varlist)
            finally:
                self.kernel.elementUnref(el)
        return True

    def __validate(self):
        self.__timeout = None
        self.validate()
        return False

    def __entry_changed(self, entry):
        if self.__timeout is None:
            self.__timeout = gobject.timeout_add(100, self.__validate)

    def __entry_style_set(self, entry, prev_style, eventbox):
        for state in (gtk.STATE_NORMAL, gtk.STATE_ACTIVE, gtk.STATE_PRELIGHT,
                      gtk.STATE_SELECTED, gtk.STATE_INSENSITIVE):
            eventbox.modify_bg(state, entry.get_style().base[state])

    def do_get_property(self, property):
        if property.name == 'variables-list':
            return self.__varlist
        else:
            raise AttributeError, 'unknown property %s' % property.name

    def get_text(self):
        return self.entry.get_text()
    def set_text(self, text):
        self.entry.set_text(text)
        self.__validate()

    def do_grab_focus(self):
        self.entry.grab_focus()

    def __entry_focus_change_cb (self, entry, event):
        self.queue_draw()
        return False

    def __get_borders(self):
        focus_width = self.entry.style_get_property("focus-line-width")
        interior_focus = self.entry.style_get_property("interior-focus")
        xborder = self.entry.style.xthickness;
        yborder = self.entry.style.ythickness;
        if not interior_focus:
            xborder += focus_width
            yborder += focus_width
        return xborder, yborder

    def do_size_request(self, requisition):
        requisition.width = self.border_width*2
        requisition.height = requisition.width
        self.entry.ensure_style()
        xborder, yborder = self.__get_borders()
        if self.child.flags() & gtk.VISIBLE:
            child_requisition = self.child.size_request()
            requisition.width += child_requisition[0]
            requisition.height += child_requisition[1]
        requisition.width += 2 * xborder
        requisition.height += 2 * yborder

    def do_expose_event(self, event):
        self.chain(event)
        if ((self.flags() & gtk.VISIBLE)
            and (self.flags() & gtk.MAPPED) 
            and event.window is self.window):
            self.__paint(event)

    def __paint(self, event):
        x = 0
        y = 0
        focus_width = self.entry.style_get_property("focus-line-width")
        interior_focus = self.entry.style_get_property("interior-focus")
        width, height = self.window.get_size()

        if (self.entry.flags() & gtk.HAS_FOCUS) and not interior_focus:
            x += focus_width
            y += focus_width
            width -= 2 * focus_width
            height -= 2 * focus_width

        self.entry.style.paint_flat_box(self.window, self.entry.state,
                                        gtk.SHADOW_NONE, None, self.entry,
                                        "entry_bg", 
                                        x, y, width, height)
     
        self.entry.style.paint_shadow(self.window, gtk.STATE_NORMAL, gtk.SHADOW_IN,
                                      None, self.entry, "entry",
                                      x, y, width, height)

        if (self.entry.flags() & gtk.HAS_FOCUS) and not interior_focus:
            x -= focus_width;
            y -= focus_width;
            width += 2 * focus_width;
            height += 2 * focus_width;

            self.entry.style.paint_focus(self.window, self.entry.state, None,
                                         self.entry, "entry",
                                         x, y, width, height)

    def do_size_allocate(self, allocation):
        self.allocation = allocation
        xborder, yborder = self.__get_borders()
        child_allocation = gtk.gdk.Rectangle()
        if self.flags() & gtk.REALIZED:
            child_allocation.x = self.border_width
            child_allocation.y = self.border_width;
            child_allocation.width = max(allocation.width - self.border_width * 2, 0)
            child_allocation.height = max(allocation.height - self.border_width * 2, 0)

            self.window.move_resize(allocation.x + child_allocation.x,
                                    allocation.y + child_allocation.y,
                                    child_allocation.width,
                                    child_allocation.height)
  
        child_allocation.x = self.border_width + xborder
        child_allocation.y = self.border_width + yborder
        child_allocation.width = max(allocation.width - (self.border_width + xborder) * 2, 0)
        child_allocation.height = max(allocation.height - (self.border_width + yborder) * 2, 0)

        self.child.size_allocate(child_allocation)


        
gobject.type_register(ExpressionEntry)


class VariableEntry(gtk.ComboBox):
    def __init__(self, expr_ent, expr_ent2=None):
        varlist = gtk.ListStore(str)
        gtk.ComboBox.__init__(self, varlist)
        cell = gtk.CellRendererText()
        self.pack_start(cell, True)
        self.add_attribute(cell, 'text', 0)
        self.__entry1 = expr_ent
        self.__entry2 = expr_ent2
        expr_ent.connect("notify::variables-list", self.__update_varlist, varlist)
        if expr_ent2 is not None:
            expr_ent2.connect("notify::variables-list", self.__update_varlist, varlist)

    def __update_varlist(self, entry, param, varlist):
        i = self.get_active_iter()
        if i:
            prev = varlist.get(i, 0)[0]
        else:
            prev = None
        varlist.clear()
        new_sel_iter = None

        list1 = self.__entry1.get_property("variables-list")
        if self.__entry2 is None:
            intersection = list1
        else:
            list2 = self.__entry2.get_property("variables-list")
            intersection = []
            for item in list1:
                if item in list2:
                    intersection.append(item)
        
        for var in intersection:
            i = varlist.append()
            varlist.set(i, 0, var)
            if var == prev:
                new_sel_iter = i
        if new_sel_iter is None:
            new_sel_iter = varlist.iter_nth_child(None, 0)
        if new_sel_iter is None:
            self.set_active(-1)
        else:
            self.set_active_iter(new_sel_iter)
    
    def get_value(self):
        i = self.get_active_iter()
        if i is None:
            return None
        else:
            return self.get_model().get(i, 0)[0]

    def set_value(self, value):
        if value:
            for row in self.get_model():
                if row[0] == value:
                    self.set_active_iter(row.iter)
                    break
            else:
                warnings.warn("Could not find %s in valuelist model" % value)
        

def get_console_entry():
    entry = gtk.Entry()
    entry.modify_font(pango.FontDescription("Monospace"))
    entry.show()
    return entry

def create_indenting_container(title, child):
    vbox = gtk.VBox(); vbox.show()
    vbox.set_spacing(6)
    label = gtk.Label(); label.show()
    label.set_markup('<span weight="bold">%s</span>' % title)
    label.set_alignment(0, 0.5)
    vbox.add(label)
    hbox = gtk.HBox(); hbox.show()
    vbox.add(hbox)
    label = gtk.Label('    '); label.show()
    hbox.add(label)
    hbox.add(child)
    return vbox


if __name__ == '__main__':
    import bonobo
    kernel = bonobo.get_object("OAFIID:Numexp_Kernel", "Bonobo/Unknown")
    w = gtk.Window()
    a = gtk.Alignment(xalign=0.5, yalign=0.5, xscale=1.0, yscale=0.0)
    w.set_border_width(6)
    entry = ExpressionEntry(kernel)
    def __variables_notify(entry, param):
        print entry.get_property("variables-list")
    entry.connect("notify::variables-list", __variables_notify)
    a.add(entry)
    a.show()
    entry.show()
    b = gtk.VBox(); b.show()
    b.add(a)
    b.add(gtk.Entry())
    w.add(b)
    w.show_all()
    w.connect("destroy", lambda w: gtk.main_quit())
    gtk.main()
    
