__all__ = ['GooMathML', 'GooMathMLView']

import math

if __name__ == '__main__':
    import sys
    sys.path.insert(0, "../../lib")
    import ltihooks
    ltihooks.install()
    sys.path.insert(0, "../../lib/pymathml/")
    sys.path.insert(0, "../../lib/nxpruler/python/")

import gobject
import gtk, gtk.gdk
import cairo
import pangocairo
import goocanvas

import mathml.xml.sax_parser
from mathml.backend.pangocairo_mathml import PangoCairoPlotter


resolution_mm = (gtk.gdk.screen_height() /
                 float(gtk.gdk.screen_height_mm()))
resolution_pt = resolution_mm * 0.3514598



class GooMathML(goocanvas.ItemSimple, goocanvas.Item):
    __gproperties__ = {
        'mathml': (str, None, None, '', gobject.PARAM_WRITABLE|gobject.PARAM_CONSTRUCT),
        'x': (float, None, None, -10e6, 10e6, 0, gobject.PARAM_WRITABLE|gobject.PARAM_CONSTRUCT),
        'y': (float, None, None, -10e6, 10e6, 0, gobject.PARAM_WRITABLE|gobject.PARAM_CONSTRUCT),
        'font-size': (float, "font size", "font size in points", -10e6, 10e6, 24.0,
                      gobject.PARAM_WRITABLE|gobject.PARAM_CONSTRUCT),
        }


    def __init__(self, **kwargs):
        goocanvas.ItemSimple.__init__(self, **kwargs)
        self.mathml_tree = None
        self.axis = None
        self._sax_handler = None

    def do_set_property(self, pspec, value):
        if pspec.name == 'mathml':
            self._mathml_string = value
            self.mathml_tree = None
        elif pspec.name == 'x':
            self.x = value
        elif pspec.name == 'y':
            self.y = value
        elif pspec.name == 'font-size':
            self.font_size = value
        else:
            raise AttributeError, 'unknown property %s' % pspec.name
        self.emit("changed", True)

    def do_create_view(self, canvas_view, parent_view):
        return GooMathMLView(canvas_view, parent_view, self)


class GooMathMLView(goocanvas.ItemViewSimple, goocanvas.ItemView):
    __gtype_name__ = 'GooMathMLView'

    def __init__(self, canvas_view, parent_view, item):
        assert isinstance(item, GooMathML)
        goocanvas.ItemViewSimple.__init__(self)
        self.parent_view = parent_view
        self.canvas_view = canvas_view
        self.item = item
        self.setup_accessibility()
        self.mathml_tree = None
        self.axis = None
        self._sax_handler = None
        item.connect("changed", self.__item_changed)
        item.connect("changed", goocanvas.item_view_simple_item_changed, self)

    def __item_changed(self, item, recompute_bounds):
        self.mathml_tree = None

    def do_update(self, entire_tree, cr):
        if not self.item._mathml_string:
            return

        if entire_tree or self.flags & goocanvas.ITEM_VIEW_NEED_UPDATE:
            self.flags &= ~goocanvas.ITEM_VIEW_NEED_UPDATE
            pl = PangoCairoPlotter(pangocairo.CairoContext(cr), resolution_pt)
            if self.mathml_tree is None:
                self.mathml_tree = mathml.xml.sax_parser.parseString(self.item._mathml_string, pl)
            root = self.mathml_tree
            root.x0 = 0
            root.y0 = 0
            root.setAttribute("fontsize", "%fpt" % self.item.font_size)
            root.update()

            bounds = goocanvas.Bounds()
            bounds.x1 = self.item.x
            bounds.y1 = self.item.y
            bounds.x2 = self.item.x + root.width
            bounds.y2 = self.item.y + root.height
            self.axis = root.axis
            self.item.user_bounds_to_device(cr, bounds)
            self.bounds = bounds
        
        return self.bounds

    def do_paint(self, cr, bounds, scale):
        if self.item.visibility != goocanvas.ITEM_VISIBLE:
            return
        if self.mathml_tree is None:
            return
        root = self.mathml_tree
        assert root is not None
        cr = pangocairo.CairoContext(cr)
        root.plotter.set_cairo_context(cr)
        cr.translate(self.item.x, self.item.y + root.height)
        root.x0 = 0
        root.y0 = 0
        root.render()
        root.plotter.close()

        return self.bounds

    def do_get_item_view_at(self, x, y, cr, is_pointer_event, parent_visible):
        return None


def _test(argv):
    window = gtk.Window()
    window.set_default_size(640, 600)
    window.show()
    window.connect("destroy", lambda w: gtk.main_quit())
    
    scrolled_win = gtk.ScrolledWindow()
    scrolled_win.set_shadow_type(gtk.SHADOW_IN)
    scrolled_win.show()
    window.add(scrolled_win)
    
    canvas = goocanvas.CanvasView()
    canvas.set_size_request(600, 450)
    canvas.set_bounds(0, 0, 1000, 1000)
    canvas.show()
    scrolled_win.add(canvas)
    
    ## Create the canvas model
    canvas_model = _create_canvas_model()
    canvas.set_model(canvas_model)

    gtk.main()


def _create_canvas_model():
    canvas_model = goocanvas.CanvasModelSimple()
    root = canvas_model.get_root_item()
    item = GooMathML(font_size=28)
    item.props.x = 100
    item.props.y = 300
    item.props.mathml = """
    <math>
    <mrow>
    <mfrac>
      <mn>1</mn>
      <mrow><mi>&#x03c3;</mi> <mo>&#8290;</mo>
        <msqrt><mn>2</mn><mo>&#8290;</mo><mi>&#x03c0;</mi></msqrt>
      </mrow>
    </mfrac>
    <mo>&#x2062;</mo>
    <msup>
    <mi>e</mi>
      <mrow>
      <mo>-</mo>
        <msup>
          <mrow><mo>(</mo>
            <mfrac>
              <mrow><mi>x</mi><mo>-</mo><mi>&#x00b5;</mi></mrow>
              <mrow><msqrt><mn>2</mn></msqrt><mo>&#8290;</mo><mi>&#x03c3;</mi></mrow>
            </mfrac>
          <mo>)</mo></mrow>
          <mn>2</mn>
        </msup>
      </mrow>
    </msup>
    </mrow>
    </math>
"""
    root.add_child(item)

    return canvas_model


if __name__ == "__main__":
    _test(sys.argv)
