
all_markers = []
by_name = dict()


class MarkerMeta(type):
    def __init__(cls, name, bases, dict_):
        global all_markers, by_name
        type.__init__(cls, name, bases, dict_)
        if cls.name is not None:
            all_markers.append(cls)
            by_name[cls.name] = cls

class Marker(object):
    """Abstract base class for Markers"""

    name = None

    __metaclass__ = MarkerMeta

    def draw(self, r):
        """draw(r) -> path_string
        Retuns an SVG path string that draws the marker with the radius 'r', using relative coordinates"""
        raise NotImplementedError


class MarkerX(Marker):
    """A 'diagonal cross' type (X) marker"""
    name = 'X'
    def draw(self, r):
        return "m %f,%f l %f,%f m %f,%f l %f,%f" % \
               (-r, -r, 2*r, 2*r,
                -2*r, 0, 2*r, -2*r)


class MarkerO(Marker):
    """A 'circle' type (O) marker"""
    name = 'O'
    def draw(self, r):
        return "m %f,0 a %f %f 0 1 1 0 0" % \
               (r, r, r)


class MarkerP(Marker):
    """An 'upright cross' type (+) marker"""
    name = '+'
    def draw(self, r):
        return "m %f,0 l %f,0 m %f,%f l 0 %f" % \
               (-r, 2*r, -r, -r, 2*r)
    

class MarkerB(Marker):
    """A 'box' type (square) marker"""
    name = unichr(0x025A1)
    def draw(self, r):
        return "m %f,%f l %f,%f l %f,%f l %f,%f l %f,%f" % \
               (-r, -r, 0, 2*r, 2*r, 0, 0, -2*r, -2*r, 0)


def _test():
    for marker_class in all_markers:
        marker = marker_class()
        print marker.name + ": "
        print marker.draw(1.0)

if __name__ == '__main__':
    _test()
