#! /usr/bin/env python
from __future__ import division
import re
import math
#from nxplot import *
import goocanvas
from dialogs.lineattr import Color, LineAttr, Linemode
from copy import deepcopy
import xml.dom.minidom as dom
import xml.sax.saxutils
import gtk.gdk
import gobject
# pymathml import
import mathml
import mathml.xml.minidom_parser as mathml_parser
#import mathml.backend.nxplot_mathml as mathml_renderer
#from xygraphC import ControlPoint
#import xygraphC

"""
Our OS who art in CPU, UNIX be thy name.
        Thy programs run, thy syscalls done,
        In kernel as it is in user!
"""

CONTROL_POINT_SIZE = 4


class Ticks:
    """Class to hold tick marks spacing and position values,
    already mapped to the drawing rectangle"""

    def __init__(self, start, div, subdiv, limit):
        self.start  = start        	# value of first tick mark
        self.div    = div        	# size of first-level division
        self.subdiv = subdiv        	# size of second-level division
        self.limit  = limit        	# value of last tick mark


class PlotObject(object):

    STATE_NORMAL = 0
    STATE_SELECTED = (1<<0)
    STATE_PRELIGHT = (1<<1)
    
    def __init__(self, values, plot_area):
        super(PlotObject, self).__init__()
        self.plot_area = plot_area
        self.kernel = plot_area.view.main_window.kernel
        self.canvas_item = None
        self.state = None
        self.lineattr = None # set by plotobject impl.
        self.outer_lineattr = None # computed by calc_state_lineattrs
        self.inner_lineattr = None # computed by calc_state_lineattrs
        if isinstance(values, dict):
            self.set_values(values)
        elif isinstance(values, dom.Element):
            self.from_xml(values, plot_area)
        else:
            raise TypeError("expected dictionary or DOM Element")

        self.set_state(self.STATE_NORMAL)

    def get_canvas_item(self):
        '''Return the canvas item that renderes this plot object'''
        raise NotImplementedError

    def math_range_changed(self):
        '''Called when the math coordinates range has changed in the plot area'''
        raise NotImplementedError

    def screen_range_changed(self):
        '''Called when the screen coordinates range has changed in the plot area'''
        raise NotImplementedError
        
    def get_values(self):
        return { 'label': self.caption,
                 'lineattr': deepcopy(self.lineattr),
                 }

    def set_values(self, values):
        self.caption = values['label']
        self.lineattr = values['lineattr']
        if self.state is not None:
            self.calc_state_lineattrs()

    def to_xml(self, doc):
        assert isinstance(doc, dom.Document)
        elem = doc.createElement("plotobject")
        elem.setAttribute("label", self.caption)
        elem.appendChild(self.lineattr.to_xml(doc))
        return elem

    def from_xml(self, elem, plotarea):
        assert isinstance(elem, dom.Element)
        assert elem.tagName == 'plotobject'
        self.caption  = elem.getAttribute("label").encode()
        lineattrs = elem.getElementsByTagName('lineattr')
        if lineattrs:
            self.lineattr = LineAttr()
            self.lineattr.from_xml(lineattrs[0])
        else:
            self.lineattr = plotarea.suggest_lineattr()

    def get_legend_sample(self):
        """Draw a small sample that identify the plot object.
        This will appear in the legend.

        Returns: a goocanvas.Item"""
        raise NotImplementedError
        
    ## FIXME
    def draw_legend_label(self, pl, attributes, really_draw, data=None):
        """Draw the label that identifies the plot object.
        This will appear in the legend.

        If @really_draw is FALSE, the function should return the
        (width, descender, ascender, data) of the bounding box of this
        sample plus any data that it wishes to save until rendering
        time.
        
        If @really_draw is TRUE, the label should be drawn,
        starting at (0, 0); the function should return nothing"""

        # This is default implementation that draws a simple label.
        # Specific PlotObject's may override this.
        pl.fontname(attributes.get('fontname', "Sans Regular"))
        pl.fontsize(attributes['fontsize'])
        pl.textangle(0)
        color = attributes.get('color', (0, 0, 0))
        pl.pencolor(color[0], color[1], color[2])
        if not really_draw:
            if self.caption is None: return 0, 0, 0
            ascender_max, descender_max, ascender, descender, width = \
                	  pl.labelmetrics(self.caption)
            return width, ascender, descender, None
        else:
            if self.caption is None:
                caption = str(self)
            else:
                caption = self.caption
            pl.move(0, 0)
            pl.label(HAlign.LEFT, VAlign.BASELINE, caption)
        

    ## FIXME
    def _draw_legend_label_mathml(self, mathml_input, pl, attributes,
                		  really_draw, data=None):
        """A variant of draw_legend_label, that receives a mathml
        string as input and renders it using nxplot_mathml """

        if not really_draw:
            #print "Parsing MathML string:"
            #print mathml_input
            plotter = mathml_renderer.NxplotPlotter(pl, 72)
            math = mathml_parser.parseString(mathml_input, plotter)
            math.setAttribute("fontsize", str(attributes['fontsize']))
            #mathml = nmml_parse(pl, mathml_input,
                #		{'initial_size': str(attributes['fontsize'])})
            #width, ascender, descender = mathml.get_dimmensions()
            math.update()
            return math.width, math.height, 0, math
        else:
            math = data
            math.render()

    def _label_to_mathml(self, label, substs):
        """ Converts a label that can contain %x markers to be replaced with
        mathml content to a global mathml string.
        
        @substs: a dictionary of type {'%x': mathml code without <math>, ...}
        """
        tokens = []
        start  = 0
        if not label: return "<math></math>"
        while 1:
            pos = label.find("%", start)
            if pos == -1:        	# % not found
                ltok = label[start:]
                next = None
                start = len(label)
            else:
                ltok = label[start:pos]
                try:
                    next = label[pos + 1]
                except IndexError:	# % ends the string
                    next = '%'
                start = pos + 2
            # replace leading spaces in ltok with <mspace> tokens
            s = ""
            for c in ltok:
                if c != ' ': break
                s += "<mspace width='0.5em'/>"
            text = xml.sax.saxutils.escape(ltok.strip())
            if text:
                s += "<mtext>%s</mtext>" % text
            # the same for the trailing spaces
            i = -1
            if ltok:
                while 1:
                    try:
                        c = ltok[i]
                    except IndexError:
                        break
                    i -= 1
                    if c != ' ': break
                    s += "<mspace width='0.5em'/>"
            tokens.append(s)
            if next is not None:
                if next == '%':
                    tokens.append("<mtext>%%</mtext>")
                else:
                    skey = '%' + next
                    try:
                	subst = substs[skey]
                	tokens.append(subst)
                    except KeyError:
                	subst = tokens.append("<mtext>%%%s</mtext>" % next)
            if start >= len(label): break
        return "<math>" + "".join(tokens) + "</math>"

    def get_y_range(self):
        """Should return (ymin, ymax), the vertical range limits of
        whatever it draws, or None if not supported."""
        pass

    def get_control_points(self):
        """Return a list of (x, y) pairs representing control points,
        in window coordinates. These control points will be painted to
        represented object selection."""
        # default empty list; specific plot object implementations
        # should override this function.
        return ()

    def update_control_points(self):
        if self.canvas is None: return
        for cp in self.control_points: cp.hide()
        self.control_points = []
        for x, y in self.get_control_points():
            try:
                cp = ControlPoint(int(x), int(y), CONTROL_POINT_SIZE)
            except OverflowError:
                cp = None
            if cp is not None:
                cp.set_canvas(self.canvas)
                self.control_points.append(cp)

    def set_control_points_visible(self, visible):
        for cp in self.control_points:
            if visible:
                cp.show()
            else:
                cp.hide()

    def distance_to_point(self, x, y):
        """Computes distance of point (x, y) to the plot object.  This
        is ideally, because in practice it computes the distance from
        (x, y) to the closest control point, which is good enough."""
        return xygraphC.distance_to_point(int(x), int(y), self.control_points)

    def set_state(self, state):
        if state == self.state:
            return
        self.state = state
        self.calc_state_lineattrs()
        self.draw()

    def set_selected(self, selected):
        if selected:
            self.set_state(self.state | self.STATE_SELECTED)
        else:
            self.set_state(self.state & (~self.STATE_SELECTED))

    def set_prelight(self, prelight):
        if prelight:
            self.set_state(self.state | self.STATE_PRELIGHT)
        else:
            self.set_state(self.state & (~self.STATE_PRELIGHT))

    def calc_state_lineattrs(self):
        outer_attr = deepcopy(self.lineattr)
        outer_attr.thickness = max(outer_attr.thickness + 15, outer_attr.thickness*2)
        outer_attr.dash_style = Linemode.SOLID
        
        if self.state == self.STATE_NORMAL:
            self.inner_lineattr = self.lineattr
            outer_attr.color.alpha = 0 # 0% (invisible)
            self.outer_lineattr = outer_attr
        elif self.state & self.STATE_SELECTED:
            self.inner_lineattr = self.lineattr
            outer_attr.color.alpha = 0x4000 # 25 %
            self.outer_lineattr = outer_attr
        elif self.state & self.STATE_PRELIGHT:
            attr = deepcopy(self.lineattr)
            attr.thickness = max(attr.thickness + 5, attr.thickness*1.25)
            r, g, b = attr.color.red, attr.color.green, attr.color.blue
            # increase the brightness without changing hue
            maxcolor = max(r, g, b)
            if maxcolor == 0:
                s = 0
            else:
                s = max(1.3, 65535.0 / maxcolor)
            r *= s
            g *= s
            b *= s
            # blend the color with white
            c = 0.45 # relative weight of white
            c1 = (1 - c)
            r = min(0xffff, r*c1 + 0xffff*c)
            g = min(0xffff, g*c1 + 0xffff*c)
            b = min(0xffff, b*c1 + 0xffff*c)
            attr.color = Color(int(r), int(g), int(b))
            self.inner_lineattr = attr

            outer_attr.color.alpha = 0 # 0% (invisible)
            self.outer_lineattr = outer_attr
        else:
            assert False, "invalid state %i!!!" % self.state

    def to_xml_string(self):
        """Return a XML string of object representation"""

        doc = dom.Document()
        doc.appendChild(self.to_xml(doc))
        return doc.toxml()

