#! /usr/bin/env python
import gtk
import gtk.gdk
import gobject
#from nxplot import *
from misc import PlotObject, Ticks
from nxpruler import *
from copy import *
from dialogs.objectlist import *
from plotobject_registry import *
from dialogs.lineattr import Linemode, LineAttr, Color
import math
import xml.dom.minidom as dom
import legend
import axis
import gnumexp_conf
from gettext import gettext as _
#import xygraphC
#import weakref
import cairo
import goocanvas

SCREEN_SIZE = math.hypot(gtk.gdk.screen_width(), gtk.gdk.screen_height())

_predefined_lineattrs = [
    LineAttr(thickness=5.0, dash_style=Linemode.SOLID, color=Color(0,0,0)),
    LineAttr(thickness=5.0, dash_style=Linemode.SOLID, color=Color(0,0x8000,0)),
    LineAttr(thickness=5.0, dash_style=Linemode.SOLID, color=Color(0x8000,0,0)),
    LineAttr(thickness=5.0, dash_style=Linemode.SOLID, color=Color(0,0x8000,0x8000)),
    ]

_SELECTION_XYGRAPH_TARGET = gtk.gdk.atom_intern("application/x-xygraph", True)
_SELECTION_CLIPBOARD = gtk.gdk.atom_intern("CLIPBOARD", True)
_SELECTION_TYPE_STRING = gtk.gdk.atom_intern("STRING", True)


# class ZoomAreaOverlay(NxpCanvasOverlay):
#     def __init__(self):
#         NxpCanvasOverlay.__init__(self)
#         self.connect("draw", self.__class__.real_draw)

#     def real_draw(self, drawable, gc, t):
#         gc.set_line_attributes(0, gtk.gdk.LINE_ON_OFF_DASH,
#                 	       gtk.gdk.CAP_NOT_LAST, gtk.gdk.JOIN_MITER)
#         drawable.draw_rectangle(gc, 0, self.bbox.x, self.bbox.y,
#                 		self.bbox.width,
#                 		self.bbox.height)


class _ObjectList:
    class _MyModel(PlotObjectListModel):
        def __init__(self, objlist):
            assert isinstance(objlist, _ObjectList)
            self.objlist = objlist
            PlotObjectListModel.__init__(self)

        def row_changed(self, iter, values, obj):
            """Called when a row has been changed"""
            obj.set_values(values)
            self.objlist.object_modified(obj)

        def row_added(self, iter, values):
            """Called when a row has been added. Should return the
            user_data argument to associate with the new row."""
            obj_class = type_name_to_class_mapping[values['type']]
            obj = self.objlist.object_added(obj_class, values)
            return obj

        def row_deleted(self, values, obj):
            self.objlist.object_deleted(obj)
            
        def row_selected(self, values, obj):
            self.objlist.object_selected(obj)


    def __init__(self, plotarea):
        assert isinstance(plotarea, PlotArea)
        self.plotarea     = plotarea
        self.plot_objects = []
        self.model        = self._MyModel(self)

    def object_deleted(self, object):
        self.plot_objects.remove(object)
        self.plotarea.plot_object_deleted(object)
        
    def object_added(self, obj_class, values):
        plot_object = obj_class(values, self.plotarea)
        self.plot_objects.append(plot_object)
        self.plotarea.plot_object_added(plot_object)
        return plot_object

    def add_object(self, object):
        self.plot_objects.append(object)
        self.model.add_object(object.get_values(), object)
        self.plotarea.plot_object_added(object)

    def object_modified(self, object):
        self.plotarea.plot_object_modified(object)

    def show_properties_dialog(self, obj):
        # get iterator for row containing obj
        iter = self.model.find(obj)
        assert iter is not None
        win = self.model.create_properties_dialog(iter)
        return win

    def object_selected(self, object):
        self.plotarea.select_object(object)

    def select_object(self, obj):
        self.model.select_row(self.model.find(obj))

    def delete_object(self, obj):
        self.model.remove_object(self.model.find(obj))


class PlotArea(goocanvas.Group):

    __gsignals__ = {
        "selection-changed": (gobject.SIGNAL_RUN_LAST, # signal flags
                	      None, # return type
                	      (object,)), # param types
        "plot-object-added": (gobject.SIGNAL_RUN_LAST, # signal flags
                              None, # return type
                              (object,)), # param types
        "plot-object-removed": (gobject.SIGNAL_RUN_LAST, # signal flags
                                None, # return type
                                (object,)), # param types
        "plot-object-modified": (gobject.SIGNAL_RUN_LAST, # signal flags
                                 None, # return type
                                 (object,)), # param types
        }

    def __init__(self, xyview):
        goocanvas.Group.__init__(self)
        self.plotobjectlist = None
        self.view = xyview
        self.clear()
        self.view.canvas_model.get_root_item().add_child(self)
        
        self.view.canvas_view.get_root_view().connect("button-press-event", self._on_button_press)
        
        self.range = goocanvas.Bounds(0, 0, 1, 1)
        self.grid_attr = LineAttr(color=Color("#ADD8E6"), thickness=2,
                                  dash_style=Linemode.SOLID)
        self.axis_attr = LineAttr(color=Color("blue"), thickness=5,
                		  dash_style=Linemode.SOLID)
        ## position of the tick marks in math units
        self.vticks = Ticks(0, 0, 0, 0)
        self.hticks = Ticks(0, 0, 0, 0)
        ## position of the tick marks in screen units
        ## these are computed by self._update_ticks_plunit()
        self.hticks_plunit = None
        self.vticks_plunit = None
        
        self.vticks_enabled = 0
        self.hticks_enabled = 0
        self.bbox = goocanvas.Bounds(0, 0, 0, 0)
        self.bbox_sz = 1
        self.range2bbox = cairo.Matrix()
        self.range2bbox = cairo.Matrix()
        self.select_area_mode = 0
        self.selecting_area   = 0
        #self.overlay = ZoomAreaOverlay()
        self.preselected_object = None
        self.selected_object = None
        
        self.legend = legend.Legend(self)

        #self._set_drawable(1)
        #self._set_visible(1)
        #self.set_property("interaction-mask", 0)

        self.grid = goocanvas.Path()
        self.add_child(self.grid)

        ## horizontal axis
        self.haxis = axis.HorizAxis(self)
        self.haxis.math_position = 0
        self.haxis.lineattr = self.axis_attr
        self.add_child(self.haxis)

        ## vertical axis
        self.vaxis = axis.VertAxis(self)
        self.vaxis.math_position = 0
        self.vaxis.lineattr = self.axis_attr
        self.add_child(self.vaxis)

        def update_other_axis_pos(other_axis, pspec, axis):
            axis.other_axis_position = other_axis.position
        self.haxis.connect("notify::position", update_other_axis_pos, self.vaxis)
        self.vaxis.connect("notify::position", update_other_axis_pos, self.haxis)
        self.haxis.props.position = 0
        self.vaxis.props.position = 0

        #self.connect("notify::canvas", self.__notify_canvas)
        self.view.canvas_view.connect("selection-received",
                                      self.__selection_received_cb)
        self.view.canvas_view.selection_add_target(_SELECTION_CLIPBOARD,
                                                   _SELECTION_XYGRAPH_TARGET, 0)
        self.view.canvas_view.connect("selection-get",
                                      self.__selection_get_cb)
           

#     def __notify_canvas(self, *args):
#         self.overlay.set_canvas(self.canvas)
#         self.canvas.connect("selection-received",
#                 	    self.__selection_received_cb)
#         self.canvas.selection_add_target(_SELECTION_CLIPBOARD,
#                 			 _SELECTION_XYGRAPH_TARGET, 0)
#         self.canvas.connect("selection-get",
#                 	    self.__selection_get_cb)

    def set_transient_for(self, win):
        self.view.set_transient_for(win)

    def set_haxis_enabled(self, enabled):
        self.haxis_enabled  = enabled
        self.haxis._set_visible(enabled)

    def set_vaxis_enabled(self, enabled):
        self.vaxis._set_visible(enabled)

    def set_legend_enabled(self, enabled):
        self.legend._set_visible(enabled)
    def get_legend_enabled(self):
        return self.legend.visible

    def clear(self):
        """Clears list of plot objects"""
        if self.plotobjectlist is not None:
            for obj in self.plotobjectlist.plot_objects:
                self._remove_plot_object_canvas_item(obj)
        self.plotobjectlist = _ObjectList(self)


    class MyPlotObjectListWindow(PlotObjectListWindow):
        def __init__(self, model, plotarea):
            PlotObjectListWindow.__init__(self, model)
            self.plotarea = plotarea
        def create_new_plot_object_dialog(self):
            win = PlotObjectListWindow.create_new_plot_object_dialog(self)
            self.plotarea.provide_new_plot_object_defaults(win)
            return win

    def provide_new_plot_object_defaults(self, win):
        values = win.get_values()
        values['lineattr'] = self.suggest_lineattr()
        win.set_values(values)

    def show_plot_objects_list_window(self):
        win = self.MyPlotObjectListWindow(self.plotobjectlist.model, self)
        return win

    def _remove_plot_object_canvas_item(self, plot_object):
        ## XXX stupid goocanvas API forces us to search for the item
        item_to_remove = plot_object.get_canvas_item()
        for i in range(self.get_n_children()):
            item = self.get_child(i)
            if item is item_to_remove:
                self.remove_child(i)
                break
        else:
            raise ValueError

    def plot_object_deleted(self, obj):
        self._remove_plot_object_canvas_item(obj)
        self.view.set_modified(1)

    def delete_plot_object(self, obj):
        if self.selected_object is obj:
            self.select_object(None)
        if self.preselected_object is obj:
            self.preselect_object(None)
        self.plotobjectlist.delete_object(obj)
        self.view.set_modified(1)
        self.emit("plot-object-removed", obj)
        #self.update_legend()

    def plot_object_modified(self, obj):
        #self.update_legend()
        self.view.set_modified(1)
        self.emit("plot-object-modified", obj)

    def plot_object_added(self, plot_object):
        assert isinstance(plot_object, PlotObject)
        canvas_item = plot_object.get_canvas_item()
        canvas_item.set_data("gnumexp-plot-object", plot_object)
        self.add_child(canvas_item)

        item_view = self.view.canvas_view.get_item_view(canvas_item)
        item_view.connect("enter_notify_event", self._plot_object_enter_notify)
        item_view.connect("leave_notify_event", self._plot_object_leave_notify)
        item_view.connect("button_press_event", self._plot_object_button_press)
        
        self.legend.raise_(None)
        plot_object.math_range_changed()
        plot_object.screen_range_changed()
        self.view.set_modified(1)
        self.emit("plot-object-added", plot_object)
        #self.update_legend()

    def add_plot_object(self, obj):
        assert isinstance(obj, PlotObject)
        self.plotobjectlist.add_object(obj)

    def set_geometry(self, x1, y1, x2, y2):
        '''Sets the position/size in screen (pixel) units'''
        self.bbox = goocanvas.Bounds(x1, y1, x2, y2)
        self._update_range2bbox()
        for obj in self.get_plot_objects():
            obj.screen_range_changed()
        self.legend.update_position()
        self.haxis.draw()
        self.vaxis.draw()

    # <<<axis interface>>>
    def axis_geometry_update_request(self, axis):
        x1, x2, y1, y2 = axis.geometry_request()
        if axis is self.haxis:
            axis.configure(goocanvas.Bounds(self.bbox.x1, y1, self.bbox.x2, y2),
                	   Interaction.MOVE_RESIZE)
        elif axis is self.vaxis:
            axis.configure(goocanvas.Bounds(x1, self.bbox.y1, x2, self.bbox.y2),
                	   Interaction.MOVE_RESIZE)
        else: assert 0

    def __configure_haxis(self):
        x1, x2, y1, y2 = self.haxis.geometry_request()
        self.haxis.configure(goocanvas.Bounds(self.bbox.x1, y1, self.bbox.x2, y2),
                	     Interaction.MOVE_RESIZE)

    def __configure_vaxis(self):
        x1, x2, y1, y2 = self.vaxis.geometry_request()
        self.vaxis.configure(goocanvas.Bounds(x1, self.bbox.y1, x2, self.bbox.y2),
                	     Interaction.MOVE_RESIZE)

    def haxis_position_changed(self, new_pos):
        self.haxis.pos_fn = self.bbox2range_y(new_pos)

    def vaxis_position_changed(self, new_pos):
        self.vaxis.pos_fn = self.bbox2range_x(new_pos)

    def _update_haxis_pos(self):
        self.haxis.props.position = self.range2bbox_y(self.haxis.math_position)
        self.haxis.draw()

    def _update_vaxis_pos(self):
        self.vaxis.props.position = self.range2bbox_x(self.vaxis.math_position)
        self.vaxis.draw()

    def set_vaxis_position(self, pos_fn):
        self.vaxis.math_position = pos_fn
        self._update_vaxis_pos()

    def set_haxis_position(self, pos_fn):
        self.haxis.pos_fn = pos_fn
        self._update_haxis_pos()
        
    # <<<>>>
    def set_range(self, x1, y1, x2, y2):
        '''Sets the "math space" range'''
        #print "set_range", x1, y1, x2, y2
        assert x2 > x1
        assert y2 > y1
        self.range = goocanvas.Bounds(x1, y1, x2, y2)
        self._update_range2bbox()
        for obj in self.get_plot_objects():
            obj.math_range_changed()

        ## haxis update
        self.haxis.ticks = self.hticks
        self.haxis.ticks_plunit = self.hticks_plunit
        self.haxis.draw()

        ## vaxis update
        self.vaxis.ticks = self.vticks
        self.vaxis.ticks_plunit = self.vticks_plunit
        self.vaxis.draw()

    def get_plot_objects(self):
        return tuple(self.plotobjectlist.plot_objects)
        
    def _update_range2bbox(self):
        if self.bbox is None:
            return
        
        self.range2bbox = cairo.Matrix()
        self.range2bbox.translate(self.bbox.x1, self.bbox.y2)
        self.range2bbox.scale((self.bbox.x2 - self.bbox.x1) / (self.range.x2 - self.range.x1),
                              (self.bbox.y1 - self.bbox.y2) / (self.range.y2 - self.range.y1))
        self.range2bbox.translate(-self.range.x1, -self.range.y1)

        self.bbox2range = cairo.Matrix(*tuple(self.range2bbox)) # copy!
        try:
            self.bbox2range.invert()
        except cairo.Error:
            self.bbox2range = cairo.Matrix()
        
        self.bbox_sz = math.hypot(self.bbox.x2 - self.bbox.x1, self.bbox.y2 - self.bbox.y1)
        self.width_scale = self.bbox_sz/SCREEN_SIZE
        if self.bbox_sz:
            self._update_ticks_plunit()
            self._update_grid()
        self._update_haxis_pos()
        self._update_vaxis_pos()

    # <<<Conversion from function (math) to canvas (pixel) units>>>
    def range2bbox_x(self, x):
        x, _ = self.range2bbox.transform_point(x, 0)
        return x

    def range2bbox_y(self, y):
        _, y = self.range2bbox.transform_point(0, y)
        return y

    # <<<Conversion from canvas (pixel) to function (math) units>>>
    #def bbox2range_x(self, x):
    # return (x - self.bbox.x1) / self.range2bbox_sx + self.range.x1

    #def bbox2range_y(self, y):
    #    return (y - self.bbox.y1) / self.range2bbox_sy + self.range.y1

    def _update_ticks_plunit(self):
        if self.hticks is None or self.hticks is None:
            return
        h_start, v_start = self.range2bbox.transform_point(self.hticks.start, self.vticks.start)
        h_limit, v_limit = self.range2bbox.transform_point(self.hticks.limit, self.vticks.limit)
        h_div, v_div = self.range2bbox.transform_distance(self.hticks.div, self.vticks.div)
        h_subdiv, v_subdiv = self.range2bbox.transform_distance(self.hticks.subdiv, self.vticks.subdiv)
        self.hticks_plunit = Ticks(start=h_start, limit=h_limit,
                                   div=h_div, subdiv=h_subdiv)
        self.vticks_plunit = Ticks(start=v_start, limit=v_limit,
                                   div=v_div, subdiv=v_subdiv)
        
    def set_vticks(self, start, div, subdiv, limit):
        #print "set_vticks", start, limit
        self.vticks = Ticks(start, div, subdiv, limit)
        self._update_ticks_plunit()
        self._update_grid()
        # vaxis update
        #self.vaxis.ticks = self.vticks
        #self.vaxis.ticks_plunit = self.vticks_plunit
        #self.vaxis.queue_draw()

    def set_hticks(self, start, div, subdiv, limit):
        self.hticks = Ticks(start, div, subdiv, limit)
        self._update_ticks_plunit()
        self._update_grid()

    # << Grid >>

    def _draw_vticks(self):
        start  = self.vticks_plunit.start
        div    = self.vticks_plunit.div
        if div == 0:
            return ''
        subdiv = self.vticks_plunit.subdiv
        limit  = self.vticks_plunit.limit
        x1 = self.bbox.x1 + 0.5
        x2 = self.bbox.x2 + 0.5
        y = start - div + 0.5
        path = []
        while y >= limit:
            if y <= start:
                path.append("M %f %f L %f %f Z" % (x1, y, x2, y))
            y += div
        return ' '.join(path)

    def _draw_hticks(self):
        start  = self.hticks_plunit.start
        div    = self.hticks_plunit.div
        if div == 0:
            return ''
        subdiv = self.hticks_plunit.subdiv
        limit  = self.hticks_plunit.limit
        y1 = self.bbox.y1 + 0.5
        y2 = self.bbox.y2 + 0.5
        x = start - div + 0.5
        path = []
        while x < limit:
            if x >= start:
                path.append("M %f %f L %f %f Z" % (x, y1, x, y2))
            x += div
        return ' '.join(path)

    def _update_grid(self):
        self.grid_attr.apply_to_goo_canvas_item(self.grid, 100/self.bbox_sz)
        vticks = self._draw_vticks()
        hticks = self._draw_hticks()
        self.grid.props.data = '\n'.join((vticks, hticks))
        


    # << XML load/save >>
    def to_xml(self, doc):
        elem = doc.createElement("plotarea")
        elem.setAttribute("x1", str(self.range.x1))
        elem.setAttribute("x2", str(self.range.x2))
        elem.setAttribute("y1", str(self.range.y1))
        elem.setAttribute("y2", str(self.range.y2))
        for obj in self.get_plot_objects():
            elem1 = obj.to_xml(doc)
            elem.appendChild(elem1)
        elem.appendChild(self.vaxis.to_xml(doc))
        elem.appendChild(self.haxis.to_xml(doc))
        return elem

    def from_xml(self, elem, replace=1):
        assert isinstance(elem, dom.Element)
        assert elem.tagName == 'plotarea'
        if replace:
            self.clear()
        ## load vertical axis properties, if present.
        vaxis = elem.getElementsByTagName(axis.VertAxis.axis_xml_name)
        if vaxis:
            self.vaxis.from_xml(vaxis[0])
        ## load horizontal axis properties, if present.
        haxis = elem.getElementsByTagName(axis.HorizAxis.axis_xml_name)
        if haxis:
            self.haxis.from_xml(haxis[0])
        ## load all plot objects.
        for plobj in elem.getElementsByTagName('plotobject'):
            plotobject = create_plot_object_from_xml(plobj, self)
            self.add_plot_object(plotobject)

    def get_y_range(self):
        ymin, ymax = None, None
        for obj in self.get_plot_objects():
            mM = obj.get_y_range()
            if mM is None: continue
            m, M = mM
            if ymin is None:
                ymin = m
                ymax = M
            else:
                if m < ymin: ymin = m
                if M > ymax: ymax = M
        if ymin is not None:
            return ymin, ymax

    def suggest_lineattr(self):
        minimum_distance = 1
        for lineattr in _predefined_lineattrs:
            minimum = None
            for existing in [obj.lineattr for obj in self.get_plot_objects()]:
                distance = LineAttr.distance(lineattr, existing)
                if minimum is None: minimum = distance
                else:               minimum = min(minimum, distance)
            if minimum > minimum_distance:
                return lineattr
        #print "Warning: no predefined lineattr is different enough"
        return _predefined_lineattrs[0]

    def preselect_object(self, obj):
        if obj is self.preselected_object:
            return
        if self.preselected_object is not None:
            self.preselected_object.set_prelight(False)
        self.preselected_object = obj
        if self.preselected_object is not None:
            self.preselected_object.set_prelight(True)

    def _plot_object_enter_notify(self, item_view, target, event):
        plot_object = item_view.get_item().get_data("gnumexp-plot-object")
        self.preselect_object(plot_object)

    def _plot_object_leave_notify(self, item_view, target, event):
        self.preselect_object(None)

    def display_preselected_properties(self, widget, obj):
        win = self.plotobjectlist.show_properties_dialog(obj)
        self.set_transient_for(win.dialog)
        
    def __menu_delete_plot_object(self, widget, obj):
        self.delete_plot_object(obj)

    def _on_button_press(self, item_view, target, event):
        self.select_object(None)
        return True

    def _plot_object_button_press(self, item_view, target, event):
        if self.preselected_object is None:
            if event.button == 1:
                self.select_object(None)
            return False

        if event.button == 1:
            self.select_object(self.preselected_object)
            return True

        elif event.button == 3: 

            menu = gtk.Menu()

            # Properties
            item = gtk.ImageMenuItem(stock_id=gtk.STOCK_PROPERTIES)
            item.show()
            item.connect("activate", self.display_preselected_properties,
                	 self.preselected_object)
            menu.append(item)
            # Delete
            item = gtk.ImageMenuItem(stock_id=gtk.STOCK_DELETE)
            item.show()
            item.connect("activate", self.__menu_delete_plot_object,
                	 self.preselected_object)
            menu.append(item)

            menu.popup(parent_menu_shell=None, parent_menu_item=None,
                       func=None, button=event.button,
                       activate_time=event.time)
            return True
        else:
            return False

    def select_object(self, obj):
        if obj is self.selected_object:
            return
        if self.selected_object is not None:
            self.selected_object.set_selected(0)
        self.selected_object = obj
        if self.selected_object is not None:
            self.selected_object.set_selected(1)
            self.plotobjectlist.select_object(self.selected_object)
        self.emit("selection-changed", self.selected_object)

    def clear_selected(self):
        if self.selected_object is not None:
            self.delete_plot_object(self.selected_object)

    def selected_properties(self):
        assert self.selected_object is not None
        win = self.plotobjectlist.show_properties_dialog(self.selected_object)
        self.set_transient_for(win.dialog)

    ## --- copy-paste ---
    __clipboard = None
    def edit_cut(self):
        assert self.selected_object is not None
        self.__clipboard = self.selected_object.to_xml_string()
        self.delete_plot_object(self.selected_object)
        rv = self.view.canvas_view.selection_owner_set(_SELECTION_CLIPBOARD,
                                                       time=0L)

    def edit_copy(self):
        assert self.selected_object is not None
        self.__clipboard = self.selected_object.to_xml_string()
        rv = self.view.canvas_view.selection_owner_set(_SELECTION_CLIPBOARD,
                                                       time=0L)
        assert rv, "Failed to claim ownership of the clipboard!"

    def __selection_received_cb(self, widget, selection_data, data):
        #print "selection-received:", selection_data, data
        #print "targets:", selection_data.get_targets()
        xml = selection_data.get_text()
        if xml is None:
            print "Received empty selection; ignoring."
            return
        doc = dom.parseString(xml)
        doc.getElementsByTagName("plotobject")
        for plobj in doc.getElementsByTagName('plotobject'):
            plotobject = create_plot_object_from_xml(plobj, self)
            self.add_plot_object(plotobject)

    def __selection_get_cb(self, widget, selection_data, info, time):
        assert self.__clipboard is not None
        print "*** selection-get:", self.__clipboard
        selection_data.set(_SELECTION_TYPE_STRING, 8, self.__clipboard)

    def edit_paste(self):
        self.view.canvas_view.selection_convert(_SELECTION_CLIPBOARD,
                                                _SELECTION_XYGRAPH_TARGET,
                                                time=0L)
        # __selection_received_cb will be called next

gobject.type_register(PlotArea)

