from misc import *
#from kernel import *
from plotobject_registry import *
from dialogs.plotobject import PlotObjectDialog
from functionobject import Function
import warnings
import dialogs.util
import markers
import xml.dom.minidom as dom
import ORBit, CORBA
def N_(message): return message

LEGEND_SAMPLE_SIZE = 8
assert LEGEND_SAMPLE_SIZE <= 15


class ScatterObjectDialog(PlotObjectDialog):
    def __init__(self, kernel, editmode):
        PlotObjectDialog.__init__(self, "scatter plot object dialog", editmode)
        #self.name = "Scatter"
        self.expression_wid = dialogs.util.ExpressionEntry(kernel)
        self.expression_wid.show()
        self.xml.get_widget("expression container").add(self.expression_wid)
        self.xml.get_widget("expression label").set_mnemonic_widget(self.expression_wid)
        self.marker = markers.all_markers[0].name
        self._update_markers_menu()
        self.size_adj = self.xml.get_widget("scatter size").get_adjustment()
        
    def get_values(self):
        rv = PlotObjectDialog.get_values(self)
        rv.update({
            'type': "Scatter",
            'contents': {
                'expression': self.expression_wid.get_text(),
                'marker':     self._get_selected_marker_from_menu().name,
                'size':       self.size_adj.get_value(),
                }
            })
        return rv

    def set_values(self, values):
        PlotObjectDialog.set_values(self, values)
        values = values['contents']
        self.expression_wid.set_text(values['expression'])
        self.marker = values['marker']
        self._update_markers_menu()
        self.size_adj.set_value(values['size'])

    def _update_markers_menu(self):
        menu, history = self._create_markers_menu(self.marker)
        opmenu = self.xml.get_widget("marker opmenu")
        opmenu.set_menu(menu)
        opmenu.set_history(history)

    def _get_selected_marker_from_menu(self):
        opmenu = self.xml.get_widget("marker opmenu")
        menu = opmenu.get_menu()
        item = menu.get_children()[opmenu.get_history()]
        return item.get_data("marker")

    def _create_markers_menu(self, selected_item):
        menu = gtk.Menu()
        index = -1
        history = 0
        for marker in markers.all_markers:
            index += 1
            item = gtk.MenuItem(marker.name)
            menu.append(item)
            item.show()
            item.set_data("marker", marker)
            if marker.name == selected_item:
                history = index
        return menu, history

    def validate(self):
        '''Make a quick validation (just see if there are no missing fields)
        If not valid, give focus to the invalid widget and return False,
        else return True'''

        if not self.expression_wid.validate():
            self.expression_wid.grab_focus()
            return False
        ## FIXME: do more detailed validation
        
        return True


class Scatter(PlotObject):

    dialog_class = ScatterObjectDialog

    def __init__(self, values, plot_area):
        self.expression = None
        self.variable = None
        self.tree = None
        self.bbox = None
        self.samples = None
        self.marker_size = 0
        self.mathml = '<math><mtext></mtext></math>'
        PlotObject.__init__(self, values, plot_area)

    def get_values(self):
        values = PlotObject.get_values(self)
        values['contents'] = { 'expression': self.expression,
                	       'marker':     self.marker.name,
                	       'size': 	     self.marker_size,
                	       }
        values['type'] = 'Scatter'
        return values

    def get_canvas_item(self):
        '''Return the canvas item that renderes this plot object'''
        if self.canvas_item is None:
            self.inner_path = goocanvas.Path()
            self.outer_path = goocanvas.Path(pointer_events=(goocanvas.EVENTS_STROKE_MASK
                                                             |goocanvas.EVENTS_FILL_MASK
                                                             |goocanvas.EVENTS_PAINTED_MASK))
            self.canvas_item = goocanvas.Group()
            self.canvas_item.add_child(self.outer_path)
            self.canvas_item.add_child(self.inner_path)
        return self.canvas_item

    def math_range_changed(self):
        '''Called when the math coordinates range has changed in the plot area'''
        self.update_samples()
        self.draw()

    def screen_range_changed(self):
        '''Called when the screen coordinates range has changed in the plot area'''
        self.draw()


    def set_values(self, values):
        PlotObject.set_values(self, values)
        values = values['contents']
        self.expression = values['expression']
        self.set_expression(self.expression)
        self.marker_size = values['size']

        name = values['marker']
        try:
            self.marker = markers.by_name[name]
        except KeyError:
            warnings.warn("Marker name '%s' is not valid!" % (values['marker'],))
        self.update_samples()
        self.__update_mathml()
        self.draw()
        
    def __del__(self):
        if self.tree is not None:
            self.kernel.kernel.elementUnref(self.tree)
        self.tree = None

    def update_samples(self):
        try:
            self.sample()
        except CORBA.UserException, ex:
            self.samples = None
            err = str(ex.__class__) + ": " + ex.detail
            dialog = gtk.MessageDialog(
                flags          = gtk.DIALOG_DESTROY_WITH_PARENT,
                type           = gtk.MESSAGE_ERROR,
                buttons        = gtk.BUTTONS_OK,
                message_format = err)
            dialog.set_property("has-separator", False)
            self.view.set_transient_for(dialog)
            dialog.run()
            dialog.destroy()
            return
        except Exception, ex:
            self.samples = None
            dialog = gtk.MessageDialog(
                flags          = gtk.DIALOG_DESTROY_WITH_PARENT,
                type           = gtk.MESSAGE_ERROR,
                buttons        = gtk.BUTTONS_OK,
                message_format = "Unknown error while sampling function:\n" +
                                 str(ex.__class__))
            self.view.set_transient_for(dialog)
            dialog.run()
            dialog.destroy()
            raise
        #self.queue_draw()

        
    def set_range(self, x1, y1, x2, y2):
        PlotObject.set_range(self, x1, y1, x2, y2)
        self.update_samples()

    def set_expression(self, expression):
        self.expression = expression
        self.update_samples()
            

    def sample(self):
        tree = self.kernel.kernel.parseString(self.expression)
        el = self.kernel.kernel.elementEval(tree)
        self.kernel.kernel.elementUnref(tree)
        res = self.kernel.kernel.elementAsAny(el).value()
        self.kernel.kernel.elementUnref(el)
        self.samples = map(lambda any: any.value(), res)
        self.samples = map(lambda pair: (pair[0].value(), pair[1].value()),
                	   self.samples)


    def draw(self):
        if self.samples is None or self.state is None:
            return
        self.get_canvas_item()

        x1 = None
        y1 = None
        path = []
        marker = self.marker()
        r = self.calc_marker_radius(self.marker_size)
        for (x, y) in self.samples:
            xs, ys = self.plot_area.range2bbox.transform_point(x, y)
            path.append("M %f,%f %s" % (xs, ys, marker.draw(r)))
        path_data = '\n'.join(path)
        
        for lineattr, canvas_path in ((self.outer_lineattr, self.outer_path),
                                      (self.inner_lineattr, self.inner_path)):
            if lineattr.color.alpha == 0:
                canvas_path.props.visibility = goocanvas.ITEM_INVISIBLE
            else:
                canvas_path.props.visibility = goocanvas.ITEM_VISIBLE
            lineattr.apply_to_goo_canvas_item(canvas_path, self.plot_area.width_scale)
            canvas_path.props.data = path_data

    def calc_marker_radius(self, size):
        return size*self.plot_area.width_scale*3

    def get_legend_sample(self):
        sample = goocanvas.Path()
        self.lineattr.apply_to_goo_canvas_item(sample, self.plot_area.width_scale)
        sample.props.data = "M %f,%f %s" % \
                            (15, 0, self.marker().draw(LEGEND_SAMPLE_SIZE))
        return sample

    def __update_mathml(self):
        try:
            el = self.kernel.kernel.parseString(self.expression)
            expression_mathml = self.kernel.kernel.elementToMathML(el)
            self.kernel.kernel.elementUnref(el)
        except ex:
            expression_mathml = '<merror>error converting expression to MathML: %s</merror>' % str(ex)
        expression_mathml = expression_mathml.decode("utf-8")
        self.mathml = self._label_to_mathml(self.caption,
                			    {'%e': expression_mathml})

    def to_xml(self, doc):
        assert isinstance(doc, dom.Document)
        elem = PlotObject.to_xml(self, doc)
        elem.setAttribute("type", "Scatter")
        expr = doc.createElement("expression")
        expr.appendChild(doc.createTextNode(self.expression))
        elem.appendChild(expr)
        marker = doc.createElement("marker")
        marker.setAttribute("size", str(self.marker_size))
        marker.setAttribute("type", self.marker.name)
        elem.appendChild(marker)
        return elem

    def from_xml(self, elem, plotarea):
        PlotObject.from_xml(self, elem, plotarea)
        assert elem.getAttribute("type") == 'Scatter'
        expr = elem.getElementsByTagName('expression')[0]
        self.expression = ""
        for node in expr.childNodes:
            if node.nodeType == node.TEXT_NODE:
                self.expression += node.data
        self.expression = self.expression.strip().encode()
        marker = elem.getElementsByTagName('marker')[0]
        self.marker_size = float(marker.getAttribute("size"))
        marker_name = marker.getAttribute("type")
        for marker in markers.all_markers:
            if marker.name == marker_name:
                self.marker = marker
                break
        else:
            # Invalid marker name in XML file -> use the default one
            self.marker = markers.MarkerX
        self.__update_mathml()
            




type_name_to_class_mapping[N_('Scatter')] = Scatter
__all__ = ['Scatter']
#print "'Scatter' plot object has been registered"


