#! /usr/bin/env python

if __name__ == '__main__':
    import sys
    sys.path.insert(0, "../../lib")
    import ltihooks
    ltihooks.install()
    sys.path.insert(0, "../../lib/pymathml/")
    sys.path.insert(0, "../../lib/nxpruler/python/")

import gtk
#from nxplot import *
#from misc import *
from nxpruler import *
from copy import deepcopy, copy
from dialogs.objectlist import *
from dialogs.util import *
from plotobject_registry import *
from plotarea import PlotArea
import xml.dom.minidom as dom
#import legend
#import axis
#import xyconsts
import gnumexp_conf
from gettext import gettext as _
#import xygraphC
import goocanvas
import gconf
import gzip
import gnomevfs
import cairo
import pangocairo

import functionobject
import scatterobject
import parametricobject


 ###############
### # # # # # ####
## # # # # # # ####
### # # # # # ####
 ###############

page_setup = None
settings = None

ui_info = \
"""
<ui>
  <menubar name='MenuBar'>

    <menu action='FileMenu'>
      <placeholder name='FileAdditions'>
        <menuitem action='FilePrint'/>
        <menuitem action='FileExport'/>
      </placeholder>
    </menu>

    <menu action='EditMenu'>
      <menuitem action='EditCut'/>
      <menuitem action='EditCopy'/>
      <menuitem action='EditPaste'/>
      <menuitem action='EditClear'/>
      <separator/>
      <menuitem action='EditInsertFunction'/>
      <menuitem action='EditInsertParametric'/>
      <menuitem action='EditInsertScatter'/>
      <separator/>
      <menuitem action='EditProperties'/>
      <menuitem action='EditVAxis'/>
      <menuitem action='EditHAxis'/>
    </menu>
    <menu action='ViewMenu'>
      <menuitem action='ViewAxis'/>
      <menuitem action='ViewGrid'/>
      <menuitem action='ViewLegend'/>
      <separator/>
      <menuitem action='ViewZoomIn'/>
      <menuitem action='ViewZoomOut'/>
      <menuitem action='ViewZoomArea'/>
      <separator/>
      <menuitem action='ViewRefresh'/>
      <menuitem action='ViewObjectList'/>
    </menu>
  </menubar>
  <toolbar name='ToolBar'>
    <toolitem action='EditProperties'/>
    <toolitem action='EditCut'/>
    <toolitem action='EditCopy'/>
    <toolitem action='EditPaste'/>
    <toolitem action='EditClear'/>
    <separator/>
    <toolitem action='ViewZoomIn'/>
    <toolitem action='ViewZoomOut'/>
    <toolitem action='ViewZoomArea'/>
    <separator/>
    <toolitem action='ViewAxis'/>
    <toolitem action='ViewGrid'/>
    <toolitem action='ViewLegend'/>
  </toolbar>
</ui>
"""


GRID_TOGGLE_KEY   = "/apps/gnumexp/XYGraph/grid_enabled"
AXIS_TOGGLE_KEY   = "/apps/gnumexp/XYGraph/axis_enabled"
LEGEND_TOGGLE_KEY = "/apps/gnumexp/XYGraph/legend_enabled"



class FileFormatVersion:
    def __init__(self, string):
        self.version = [int(x) for x in string.split('.')]
    def __str__(self):
        return '.'.join(map(str, self.version))
    def __cmp__(self, other):
        return cmp(self.version, other.version)

this_file_format_version = FileFormatVersion('0.1')


class RectangleSelector(gobject.GObject):
    __gsignals__ = {
        "selected": (gobject.SIGNAL_RUN_LAST, # signal flags
                     None, # return type
                     ()), # param types
        }
    
    def __init__(self, group_view, button_press_event):
        assert isinstance(group_view, goocanvas.GroupView)
        gobject.GObject.__init__(self)
        self.group_view = group_view
        self.x1 = button_press_event.x
        self.y1 = button_press_event.y
        self.x2 = self.x1
        self.y2 = self.y1

        ## translucent fill makes it a lot slower... :|
        #gdkcolor = gtk.gdk.color_parse("khaki")
        #alpha = 0x8000
        #fill_color_rgba = ((int(gdkcolor.red/256) << 24)
        #                   | (int(gdkcolor.green/256) << 16)
        #                   | (int(gdkcolor.blue/256) << 8)
        #                   | (int(alpha/256)))

        self.rectangle = goocanvas.Rect(x=self.x1, y=self.y1,
                                        width=0, height=0,
                                        line_width=3.0,
                                        radius_x=4.0,
                                        radius_y=4.0,
                                        stroke_color="black",
                                        #fill_color_rgba=fill_color_rgba,
                                        pointer_events=goocanvas.EVENTS_NONE)
        self.group_view.get_item().add_child(self.rectangle)
        self.rectangle.raise_(None)
        
        self._motion_notify_handler = self.group_view.connect("motion-notify-event", self._on_motion_notify)
        self._button_release_handler = self.group_view.connect("button-release-event", self._on_button_release)

    def _cleanup(self):
        if self._motion_notify_handler is not None:
            self.group_view.disconnect(self._motion_notify_handler)
            self._motion_notify_handler = None

        if self._button_release_handler is not None:
            self.group_view.disconnect(self._button_release_handler)
            self._button_release_handler = None
            
        if self.rectangle is not None:
            #_canvas_group_remove_child(self.group_view.get_item(), self.rectangle)
            self.group_view.get_item().remove_child(self.rectangle)
            self.group_view = None
            self.rectangle = None

    def cancel(self):
        self._cleanup()

    def _on_button_release(self, view, target, event):
        self._cleanup()
        self.emit("selected")
        return True

    def _on_motion_notify(self, rect_view, target, event):
        self.x2 = event.x
        self.y2 = event.y

        l = [self.x1, self.x2]
        l.sort()
        x1, x2 = l
        
        l = [self.y1, self.y2]
        l.sort()
        y1, y2 = l

        self.rectangle.props.x = x1
        self.rectangle.props.y = y1
        self.rectangle.props.width = x2 - x1
        self.rectangle.props.height = y2 - y1
        
        return True


class XYGraphView(gobject.GObject):
    __gproperties__ = { 'modified' :
                        (bool, 
                         'document modified status',
                         'becomes true when the document is modified, and false when the document is saved',
                         False,
                         gobject.PARAM_READABLE)
                        }

    def __init__(self, main_window):
        gobject.GObject.__init__(self)
        self.main_window = main_window

        self.__modified = 0
        self.__modified_ignore = 0
        self._zoom_area_mode = None
        self._rectangle_selector = None
        
        conf = gconf.client_get_default()

        self.axis_enabled = conf.get_bool(AXIS_TOGGLE_KEY)
        self.legend_enabled = conf.get_bool(LEGEND_TOGGLE_KEY)
        
        self.canvas_view = goocanvas.CanvasView()
        self.canvas_view.add_events(gtk.gdk.POINTER_MOTION_MASK
                                    | gtk.gdk.POINTER_MOTION_HINT_MASK)

        self.canvas_view.show()
        self.canvas_view.set_bounds(0, 0, 0, 0)
        self.canvas_view.set_size_request(400, 300)
        self.canvas_model = goocanvas.CanvasModelSimple()
        self.canvas_view.set_model(self.canvas_model)
        self.plot_area =  PlotArea(self)

        self.grid_enabled = conf.get_bool(GRID_TOGGLE_KEY)
        if self.grid_enabled:
            self.plot_area.grid.props.visibility = goocanvas.ITEM_VISIBLE
        else:
            self.plot_area.grid.props.visibility = goocanvas.ITEM_INVISIBLE

        self.axis_enabled = conf.get_bool(AXIS_TOGGLE_KEY)
        if self.axis_enabled:
            self.plot_area.vaxis.props.visibility = goocanvas.ITEM_VISIBLE
            self.plot_area.haxis.props.visibility = goocanvas.ITEM_VISIBLE
        else:
            self.plot_area.vaxis.props.visibility = goocanvas.ITEM_INVISIBLE
            self.plot_area.haxis.props.visibility = goocanvas.ITEM_INVISIBLE

        self.legend_enabled = conf.get_bool(LEGEND_TOGGLE_KEY)
        if self.legend_enabled:
            self.plot_area.legend.props.visibility = goocanvas.ITEM_VISIBLE
        else:
            self.plot_area.legend.props.visibility = goocanvas.ITEM_INVISIBLE

        #canvas.set_viewport(0, 0, xyconsts.PLOT_AREA_WIDTH, xyconsts.PLOT_AREA_HEIGHT)
        #canvas.set_root(self.plot_area)
        #self.plot_area.connect("selection-changed", self.__selection_changed_cb)

        self.hadj = hadj = NxpRulerAdjustment(-10, 10)
        self.vadj = vadj = NxpRulerAdjustment(-10, 10)
        # A "range_changed" signal is emmited whenever the ruler is
        #   scrolled or stretched.
        self.hadj.connect_after('range_changed', self._hrange_changed)
        self.vadj.connect_after("range_changed", self._vrange_changed)

        # table
        self.table = table = gtk.Table(3, 2, 0)
        table.show()

        # hruler
        self.hruler = hruler = NxpHRuler(self.hadj); hruler.show()
        table.attach(hruler, 1, 2, 0, 1, gtk.EXPAND|gtk.FILL, 0, 0, 0)
        self.hruler.connect('button-press-event', self.__hruler_button_press)

        # vruler
        self.vruler = vruler = NxpVRuler(vadj); vruler.show()
        table.attach(vruler, 0, 1, 1, 2, 0, gtk.EXPAND|gtk.FILL, 0, 0);
        self.vruler.connect('button-press-event', self.__vruler_button_press)

        # add canvas
        table.attach(self.canvas_view, 1, 2, 1, 2)
        self.canvas_view.connect("motion_notify_event", self._canvas_motion_notify)
        #self.canvas_view.connect("button-press-event", self._canvas_button_press)
        self.last_allocation = None
        self.canvas_view.connect_after("size-allocate", self._canvas_size_allocate)

        self.hruler.connect_after("size-allocate", self._hruler_size_allocate)
        self.vruler.connect_after("size-allocate", self._vruler_size_allocate)

        self.width, self.height = (0, 0)
        self._range_changed()

        ## Merge console UI elements
        if main_window is not None:
            mergeid = main_window.ui_manager.add_ui_from_string(ui_info)
            main_window.ui_manager.insert_action_group(self.__create_action_group(), 0)

        self.canvas_view.get_root_view().connect("button-press-event", self._canvas_button_press)

    def do_get_property(self, pspec):
        if pspec.name == 'modified':
            return self.__modified
        else:
            raise TypeError
    

    def __create_action_group(self):
        # GtkActionEntry
        entries = (

          ( "FilePrint", gtk.STOCK_PRINT,      # name, stock id
            _("Print..."), None,               # label, accelerator
            _("Print this plot"),              # tooltip
            self.on_file_print),
          ( "FileExport", None,                # name, stock id
            _("Export..."), None,              # label, accelerator
            _("Export graphics to a file"),    # tooltip
            self.on_file_export),

            ##
            ## Copy-paste
            ##
          ( "EditCut", gtk.STOCK_CUT,          # name, stock id
            None, None,                        # label, accelerator
            _("Cut the selection"),            # tooltip
            self.on_edit_cut),

          ( "EditCopy", gtk.STOCK_COPY,        # name, stock id
            None, None,                        # label, accelerator
            _("Copy the selection"),           # tooltip
            self.on_edit_copy),

          ( "EditPaste", gtk.STOCK_PASTE,      # name, stock id
            None, None,                        # label, accelerator
            _("Paste the clipboard"),          # tooltip
            self.on_edit_paste),

          ( "EditClear", gtk.STOCK_CLEAR,      # name, stock id
            None, None,                        # label, accelerator
            _("Remove the selected object"),   # tooltip
            self.on_edit_clear),

        ##
        ## Plot Objects
        ##
          ( "EditInsertFunction", None,            # name, stock id
            _("Insert _Function"), "<Control>F",   # label, accelerator
            _("Draw a function in this plot"),     # tooltip
            self.on_insert_function),
          ( "EditInsertParametric", None,                   # name, stock id
            _("Insert _Parametric"), "<Control>P",          # label, accelerator
            _("Draw a parametric function in this plot"),   # tooltip
            self.on_insert_parametric),
          ( "EditInsertScatter", None,                  # name, stock id
            _("Insert _Scatter"), "<Control><Shift>S",  # label, accelerator
            _("Draw a set of points in this plot"),     # tooltip
            self.on_insert_scatter),

        ##
        ## Properties
        ##
          ( "EditProperties", gtk.STOCK_PROPERTIES,    # name, stock id
            None, None,                                # label, accelerator
            None,               # tooltip
            self.on_edit_properties),

          ( "EditVAxis", gtk.STOCK_PROPERTIES,               # name, stock id
            _("Vertical Axis"), None,                        # label, accelerator
            _("Modify the properties of the vertical axis"), # tooltip
            self.on_edit_vaxis),

          ( "EditHAxis", gtk.STOCK_PROPERTIES,                 # name, stock id
            _("Horizontal Axis"), None,                        # label, accelerator
            _("Modify the properties of the horizontal axis"), # tooltip
            self.on_edit_haxis),

        ##
        ## Properties
        ##
          ( "ViewZoomIn", gtk.STOCK_ZOOM_IN,   # name, stock id
            None, None,                        # label, accelerator
            None,               # tooltip
            self.on_view_zoom_in),

          ( "ViewZoomOut", gtk.STOCK_ZOOM_OUT, # name, stock id
            None, None,                        # label, accelerator
            None,               # tooltip
            self.on_view_zoom_out),

          ( "ViewRefresh", gtk.STOCK_REFRESH,    # name, stock id
            None, None,                          # label, accelerator
            None,                                # tooltip
            self.on_view_refresh),

          ( "ViewObjectList", None,                             # name, stock id
            _("Show _Objects List"), "<Control>L",              # label, accelerator
            _("Show a window with the list of plot objects"),   # tooltip
            self.on_view_object_list),


          ) ## end

        # GtkToggleActionEntry
        toggle_entries = (

          ( "ViewZoomArea", gtk.STOCK_ZOOM_FIT,          # name, stock id
            _("Zoom Area"), None,                        # label, accelerator
            _("Select a rectangular area to zoom"),      # tooltip
            self.on_toggle_view_zoom_area),

          ( "ViewAxis", "gnumexp-axis",       # name, stock id
            _("_Axis"), None,                 # label, accelerator
            _("Toggle display of axis"),      # tooltip
            self.on_toggle_view_axis, self.axis_enabled),

          ( "ViewGrid", "gnumexp-grid",       # name, stock id
            _("_Grid"), None,                 # label, accelerator
            _("Toggle display of grid"),      # tooltip
            self.on_toggle_view_grid, self.grid_enabled),

          ( "ViewLegend", "gnumexp-legend",     # name, stock id
            _("_Legend"), None,                 # label, accelerator
            _("Toggle display of legend"),      # tooltip
            self.on_toggle_view_legend, self.legend_enabled),
          
         )

        # Create the menubar and toolbar
        action_group = gtk.ActionGroup("ConsoleActions")
        action_group.add_actions(entries)
        action_group.add_toggle_actions(toggle_entries)

        return action_group
        
    def on_file_print(self, action):
        global settings, page_setup

        print_ = gtk.PrintOperation()

        if settings is not None:
            print_.set_print_settings(settings)

        if page_setup is not None:
            print_.set_default_page_setup(page_setup)

        print_.connect("begin_print", self._begin_print)
        print_.connect("draw_page", self._draw_page)

        parent_window = self.canvas_view.get_toplevel()
        try:
            res = print_.run(gtk.PRINT_OPERATION_ACTION_PRINT_DIALOG, parent_window)
        except gobject.GError, ex:
            error_dialog = gtk.MessageDialog(parent_window,
                                             gtk.DIALOG_DESTROY_WITH_PARENT,
                                             gtk.MESSAGE_ERROR,
                                             gtk.BUTTONS_CLOSE,
                                             ("Error printing file:\n%s" % str(ex)))
            error_dialog.connect("response", gtk.Widget.destroy)
            error_dialog.show()
        else:
            if res == gtk.PRINT_OPERATION_RESULT_APPLY:
                settings = print_.get_print_settings()
        
    def _begin_print(self, operation, context):
        operation.set_n_pages(1)

    def on_edit_cut(self, action):
        self.plot_area.edit_cut()

    def on_edit_copy(self, action):
        self.plot_area.edit_copy()

    def on_edit_paste(self, action):
        self.plot_area.edit_paste()

    def on_edit_clear(self, action):
        self.plot_area.clear()

    def on_insert_function(self, action):
        d = self.show_add_object_dialog('Function').dialog
        d.set_transient_for(self.canvas_view.get_toplevel())

    def on_insert_parametric(self, action):
        d = self.show_add_object_dialog('Parametric').dialog
        d.set_transient_for(self.canvas_view.get_toplevel())

    def on_insert_scatter(self, action):
        d = self.show_add_object_dialog('Scatter').dialog
        d.set_transient_for(self.canvas_view.get_toplevel())

    def on_edit_properties(self, action):
        self.plot_area.selected_properties()

    def on_edit_vaxis(self, action):
        self.create_vaxis_properties_dialog()
        
    def on_edit_haxis(self, action):
        self.create_haxis_properties_dialog()

    def on_view_zoom_in(self, action):
        self.zoom_in()

    def on_view_zoom_out(self, action):
        self.zoom_out()

    def on_toggle_view_zoom_area(self, action):
        self._zoom_area_mode = action.get_active()
        if self._zoom_area_mode:
            self.canvas_view.pointer_grab(self.canvas_view.get_root_view(),
                                          #gtk.gdk.ALL_EVENTS_MASK,
                                          (gtk.gdk.POINTER_MOTION_MASK|
                                           gtk.gdk.POINTER_MOTION_HINT_MASK|
                                           gtk.gdk.ENTER_NOTIFY_MASK|
                                           gtk.gdk.LEAVE_NOTIFY_MASK|
                                           gtk.gdk.BUTTON_RELEASE_MASK|
                                           gtk.gdk.BUTTON_PRESS_MASK),
                                          gtk.gdk.Cursor(gtk.gdk.CROSSHAIR),
                                          0)
        else:
            self.canvas_view.pointer_ungrab(self.canvas_view.get_root_view(), 0)
            if self._rectangle_selector is not None:
                self._rectangle_selector.cancel()
                self._rectangle_selector = None

    def on_view_refresh(self, action):
        self.refresh()

    def on_view_object_list(self, action):
        win = self.show_plot_objects_list_window()
        self.set_transient_for(win.win)
        win.show()
        
    def on_toggle_view_axis(self, action):
        if action.get_active():
            visibility = goocanvas.ITEM_VISIBLE
        else:
            visibility = goocanvas.ITEM_INVISIBLE
        self.plot_area.vaxis.props.visibility = visibility
        self.plot_area.haxis.props.visibility = visibility
        gconf.client_get_default().set_bool(AXIS_TOGGLE_KEY, action.get_active())
        
    def on_toggle_view_grid(self, action):
        if action.get_active():
            visibility = goocanvas.ITEM_VISIBLE
        else:
            visibility = goocanvas.ITEM_INVISIBLE
        self.plot_area.grid.props.visibility = visibility
        gconf.client_get_default().set_bool(GRID_TOGGLE_KEY, action.get_active())

    def on_toggle_view_legend(self, action):
        if action.get_active():
            visibility = goocanvas.ITEM_VISIBLE
        else:
            visibility = goocanvas.ITEM_INVISIBLE
        self.plot_area.legend.props.visibility = visibility
        gconf.client_get_default().set_bool(LEGEND_TOGGLE_KEY, action.get_active())
        
    def _canvas_size_allocate(self, canvas, allocation):
        if (self.last_allocation is not None and
            self.last_allocation[0] == allocation.width and
            self.last_allocation[1] == allocation.height):
            return
        self.last_allocation = allocation.width, allocation.height
        self.canvas_view.set_bounds(0, 0, allocation.width, allocation.height)
        self.width, self.height = allocation.width, allocation.height
        self.plot_area.set_geometry(0, 0,
                                    allocation.width,
                                    allocation.height)

    def _canvas_motion_notify(self, canvas, event):
        self.hruler.event(event)
        self.vruler.event(event)

    def _rectangle_selected_cb(self, sel):
        self._rectangle_selector = None
        toggle_action = self.main_window.ui_manager.get_action("/MenuBar/ViewMenu/ViewZoomArea")
        toggle_action.set_active(False)
        x1, y1 = self.plot_area.bbox2range.transform_point(sel.x1, sel.y1)
        x2, y2 = self.plot_area.bbox2range.transform_point(sel.x2, sel.y2)
        if x1 > x2:
            x1, x2 = x2, x1
        if y1 > y2:
            y1, y2 = y2, y1
        self.set_zoom_area(x1, y1, x2, y2)

    def _canvas_button_press(self, item_view, target, event):
        if not (self._zoom_area_mode and self._rectangle_selector is None):
            return False
        self._rectangle_selector = RectangleSelector(item_view, event)
        self._rectangle_selector.connect("selected", self._rectangle_selected_cb)
        return True

    def _hruler_size_allocate(self, hruler, allocation):
        self.__modified_ignore = True
        self.hruler.compute_ticks()
        self._hrange_changed(self.hadj)
        self.__modified_ignore = False

    def _vruler_size_allocate(self, vruler, allocation):
        self.__modified_ignore = True
        self._vrange_changed(self.vadj)
        self.__modified_ignore = False

    def _range_changed(self):
        self.plot_area.set_range(self.hadj.lower, self.vadj.lower,
                		 self.hadj.upper, self.vadj.upper)
        self.set_modified(True)

    def _hrange_changed(self, adj):
        #print "Horizontal range changed: new range = [%f, %f]" % (adj.lower, adj.upper)
        self.hruler.compute_ticks()
        self.plot_area.set_hticks(self.hruler.rtick_start, self.hruler.rtick_div,
                		  self.hruler.rtick_subdiv, self.hruler.rtick_limit)
        self._range_changed()

    def _vrange_changed(self, adj):
        #print "Vertical range changed: new range = [%f, %f]" % (adj.lower, adj.upper)
        self.vruler.compute_ticks()
        self.plot_area.set_vticks(self.vruler.rtick_start, self.vruler.rtick_div,
                		  self.vruler.rtick_subdiv, self.vruler.rtick_limit)
        self._range_changed()
        
    def get_widget(self):
        return self.table
    
    def show_plot_objects_list_window(self):
        return self.plot_area.show_plot_objects_list_window()

    def show_add_object_dialog(self, objtype_name):
        win = self.plot_area.plotobjectlist.model.create_new_object_dialog(
            self.main_window.kernel.kernel,
            type_name_to_class_mapping[objtype_name].dialog_class)
        self.plot_area.provide_new_plot_object_defaults(win)
        return win

    def zoom_in(self):
        x1, y1, x2, y2 = self.hadj.lower, self.vadj.lower,\
                	 self.hadj.upper, self.vadj.upper
        dx, dy = x2 - x1, y2 - y1
        marginx, marginy = dx/4, dy/4
        x1 += marginx
        y1 += marginy
        x2 -= marginx
        y2 -= marginy
        self.hadj.set_range(x1, x2)
        self.vadj.set_range(y1, y2)

    def zoom_out(self):
        x1, y1, x2, y2 = self.hadj.lower, self.vadj.lower,\
                	 self.hadj.upper, self.vadj.upper
        dx, dy = x2 - x1, y2 - y1
        marginx, marginy = dx/2, dy/2
        x1 -= marginx
        y1 -= marginy
        x2 += marginx
        y2 += marginy
        self.hadj.set_range(x1, x2)
        self.vadj.set_range(y1, y2)

    def set_zoom_area(self, x1, y1, x2, y2):
        if x2 > x1 and y2 > y1:
            self.hadj.set_range(x1, x2)
            self.vadj.set_range(y1, y2)


    def zoom_area(self, state):
        if state:
            self.plot_area.begin_zoom_select()
        else:
            self.plot_area.cancel_zoom_select()


    def save_to(self, savefile):
        """save xml to a file-like object @savefile"""

        gzipfile = gzip.GzipFile(mode="w", fileobj=savefile)
        doc  = dom.Document()
        xy = doc.createElement("xygraph")
        xy.setAttribute("version", str(this_file_format_version))
        xy.setAttribute("creator", "gnumexp " + gnumexp_conf.VERSION)
        plarea = self.plot_area.to_xml(doc)
        xy.appendChild(plarea)
        doc.appendChild(xy)
        doc.writexml(gzipfile, addindent="    ", newl="\n")
        gzipfile.close()
        self.set_modified(False)

    def load_from(self, loadfile):
        """read xml from a file-like object @loadfile"""

        ## Try to autodetect whether the file is in gzip format or not
        ## (since certain older gnumexp versions saved in uncompressed xml)
        f = gzip.GzipFile(fileobj=loadfile)
        try:
            f.read(1)
        except IOError: # Not a gzipped file
            f.close()
            loadfile.seek(0)
            f = loadfile
        else:
            f.seek(0)

        doc = dom.parse(f)
        root = doc.documentElement
        if root.tagName != "xygraph":
            raise RuntimeError("This is not a gnumexp plot file!")

        # Check if we can read this format version
        ver = root.getAttribute('version')
        if ver:
            if FileFormatVersion(ver) > this_file_format_version:
                msg = _("This file was created with a more recent version of gnumexp.\n"\
                	"The file will be read anyway, but there will be at least"\
                	" some information loss.")
                dialog = gtk.MessageDialog(
                    flags          = gtk.DIALOG_DESTROY_WITH_PARENT,
                    type           = gtk.MESSAGE_WARNING,
                    buttons        = gtk.BUTTONS_OK,
                    message_format = msg)
                self.set_transient_for(dialog)
                dialog.show()
                dialog.connect("response", lambda d, r: d.destroy())
        
        plotareas = root.getElementsByTagName('plotarea')
        if len(plotareas) == 0:
            raise RuntimeError("File is broken: missing <plotarea> element.")
        if len(plotareas) > 1:
            raise RuntimeError("File is broken: too many <plotarea> elements.")
        pa = plotareas[0]
        self.from_xml_plotarea(pa)

    def from_xml_plotarea(self, pa, replace=1):
        self.plot_area.from_xml(pa, replace)
        self.__modified_ignore = 1
        # get x range, if any
        x1 = pa.getAttribute("x1")
        x2 = pa.getAttribute("x2")
        if x1 and x2:
            self.hadj.set_range(float(x1), float(x2))
        # get y range, if any
        y1 = pa.getAttribute("y1")
        y2 = pa.getAttribute("y2")
        if y1 and y2:
            self.vadj.set_range(float(y1), float(y2))
        self.__modified_ignore = 0
        self.set_modified(False)


    def refresh(self):
        x1, y1, x2, y2 = self.hadj.lower, self.vadj.lower,\
                	 self.hadj.upper, self.vadj.upper
        self.hadj.set_range(x1, x2)
        self.vadj.set_range(y1, y2)

    def set_transient_for(self, win):
        win.set_transient_for(self.canvas_view.get_toplevel())

    def set_modified(self, modified):
        if self.__modified_ignore:
            return
        ## if value doesn't change, ignore
        if (self.__modified and modified or
            not self.__modified and not modified):
            return
        self.__modified = modified
        self.notify("modified")
        
    def get_plot_objects(self):
        return copy(self.plot_area.plotobjectlist.plot_objects)

    def _draw_page(self, operation, context, page_nr):
        print page_nr

        cr = context.get_cairo_context()
        page_width = context.get_width()
        page_height = context.get_height()
    
        page_ewidth = page_width*.9        # effective page width
        page_eheight = page_height*.9        # effective page height
        
        canvas_width  = self.canvas_view.allocation.width
        canvas_height = self.canvas_view.allocation.height
        
        if page_width <= page_height:
            scale = page_ewidth / canvas_width
        else:
            scale = page_eheight / canvas_height

        ## destination rectangle
        dest_width  = scale*canvas_width
        dest_height = scale*canvas_height

        ## Especially with landscape mode, these dimmensions can still
        ## go outside the paper, so we may need to readjust the scale
        ## factor.
        if dest_width > page_ewidth:
            ## Need to limit by width
            scale = page_ewidth / canvas_width
            dest_width  = scale*canvas_width
            dest_height = scale*canvas_height
        elif dest_height > page_eheight:
            ## Need to limit by height
            scale = page_eheight / canvas_height
            dest_width  = scale*canvas_width
            dest_height = scale*canvas_height
        
        ## Offsets to center on page
        offset_x = page_width/2 - dest_width/2
        offset_y = page_height/2 - dest_height/2

        ## draw rectangle
        x1, y1 = offset_x, offset_y
        x2, y2 = x1 + dest_width, y1 + dest_height
        cr.rectangle(offset_x, offset_y, dest_width, dest_height)
        cr.set_line_width(page_width/300)
        cr.set_source_rgb(0, 0, 0)
        cr.stroke_preserve()

        cr.save()
        cr.clip()
        ## setup transformation
        cr.translate(offset_x, offset_y)
        cr.scale(scale, scale)

        ## draw the canvas to a printing context
        self.canvas_view.render(cr)

        cr.restore()

        ## draw rectangle
        cr.rectangle(offset_x, offset_y, dest_width, dest_height)
        cr.set_line_width(page_width/300)
        cr.set_source_rgb(0, 0, 0)
        cr.stroke()

    def _file_export_get_uri(self):
        sel = gtk.FileChooserDialog("Save...", self.canvas_view.get_toplevel(),
                                    gtk.FILE_CHOOSER_ACTION_SAVE,
                                    (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                     gtk.STOCK_SAVE, gtk.RESPONSE_OK))
        sel.set_default_response(gtk.RESPONSE_OK)
        sel.set_local_only(False)
        sel.set_do_overwrite_confirmation(True)
        if self.main_window.document_uri is None:
            sel.set_current_name(_("Unnamed.pdf"))
        else:
            name = gnomevfs.URI(self.main_window.document_uri).short_name
            if name.endswith(".xyg"):
                fname = name[:-4] + ".pdf"
            else:
                fname = name + ".pdf"
            sel.set_current_name(fname)
        filter = gtk.FileFilter()
        filter.set_name(_("Embedded PostScript"))
        filter.add_mime_type("image/x-eps")
        sel.add_filter(filter)

        filter = gtk.FileFilter()
        filter.set_name(_("Portable Document Graphics"))
        filter.add_mime_type("application/pdf")
        sel.add_filter(filter)

        filter = gtk.FileFilter()
        filter.set_name(_("Scalable Vector Graphics"))
        filter.add_mime_type("image/svg+xml")
        sel.add_filter(filter)

        resp = sel.run()
        if resp != gtk.RESPONSE_OK:
            sel.destroy()
            return None
        
        uri = sel.get_uri()
        sel.destroy()
        return gnomevfs.URI(uri)
        

    def on_file_export(self, action):
        uri = self._file_export_get_uri()
        if uri is None:
            return

        class GnomeVFSFile(object):
            def __init__(self, uri):
                self.handle = gnomevfs.create(uri, gnomevfs.OPEN_WRITE)
            def write(self, data):
                self.handle.write(data)
            def close(self):
                self.handle.close()

        offset_x, offset_y = 0, 0
        dest_width = self.canvas_view.allocation.width
        dest_height = self.canvas_view.allocation.height
        try:
            dummy, extension = uri.short_name.rsplit('.')
        except ValueError:
            extension = '(none)'
        else:
            extension = extension.lower()
        if extension == 'eps':
            vfsfile = GnomeVFSFile(uri)
            surface = cairo.PSSurface(vfsfile, dest_width, dest_height)
        elif extension == 'pdf':
            vfsfile = GnomeVFSFile(uri)
            surface = cairo.PDFSurface(vfsfile, dest_width, dest_height)
        elif extension == 'svg':
            vfsfile = GnomeVFSFile(uri)
            surface = cairo.SVGSurface(vfsfile, dest_width, dest_height)
        else:
            dialog = gtk.MessageDialog(parent  = self.canvas_view.get_toplevel(),
                		       flags   = gtk.DIALOG_DESTROY_WITH_PARENT,
                		       type    = gtk.MESSAGE_ERROR,
                		       buttons = gtk.BUTTONS_OK,
                		       message_format = _("Unknown extension '%s' (valid extensions are 'eps' and 'pdf')"
                                                          % (extension,)))
            dialog.run()
            dialog.destroy()
            return
            
        cr = pangocairo.CairoContext(cairo.Context(surface))
        ## draw rectangle
        x1, y1 = offset_x, offset_y
        x2, y2 = x1 + dest_width, y1 + dest_height
        cr.rectangle(offset_x, offset_y, dest_width, dest_height)
        cr.set_line_width(dest_width/300)
        cr.set_source_rgb(0, 0, 0)
        cr.stroke_preserve()

        cr.save()
        cr.clip()

        ## draw the canvas to a printing context
        self.canvas_view.render(cr)

        cr.restore()

        ## draw rectangle
        cr.rectangle(offset_x, offset_y, dest_width, dest_height)
        cr.set_line_width(dest_width/300)
        cr.set_source_rgb(0, 0, 0)
        cr.stroke()

        cr.show_page()
        surface.finish()
        vfsfile.close()
        
        

    def auto_vertical_fit(self):
        mM = self.plot_area.get_y_range()
        #print "self.plot_area.get_y_range():", mM
        if mM is None: return
        ymin, ymax = mM
        dy = (ymax - ymin)/10
        if dy == 0: return
        self.vadj.set_range(ymin - dy, ymax + dy)

    def __selection_changed_cb(self, plotarea, selection):
        if self.control is not None:
            self.control.selection_changed(selection)
        

    __haxis_properties_dialog = None
    __vaxis_properties_dialog = None
    def __axis_properties_dialog_destroyed(self, dialog):
        if dialog is self.__haxis_properties_dialog:
            self.__haxis_properties_dialog = None
        elif dialog is self.__vaxis_properties_dialog:
            self.__vaxis_properties_dialog = None
        else:
            assert 0, "unknown dialog destroyed"
    def __create_axis_properties_dialog_common(self, axis_Name, axis_pos_coord, pos, lower, upper):
        dialog = gtk.Dialog(title   = _("%s axis properties") % (axis_Name,),
                	    flags   = gtk.DIALOG_DESTROY_WITH_PARENT,
                	    buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                		       gtk.STOCK_OK,     gtk.RESPONSE_ACCEPT)
                	    )
        dialog.set_default_response(gtk.RESPONSE_ACCEPT)
        dialog.set_property("has-separator", 0)
        dialog.set_border_width(12)
        self.set_transient_for(dialog)
        dialog.vbox.set_spacing(18)
        dialog.connect("destroy", self.__axis_properties_dialog_destroyed)

        # add content
        ## -> position
        hbox = gtk.HBox()
        label = gtk.Label("%s = " % axis_pos_coord)
        entry = gtk.Entry()
        entry.set_text(pos)
        hbox.add(label)
        hbox.add(entry)
        hbox.show_all()
        hbox.set_spacing(6)
        dialog.vbox.add(create_indenting_container(_("Position"), hbox))
        dialog.position_entry = entry
        entry.set_activates_default(True)

        ## -> limits
        vbox = gtk.VBox();
        vbox.show(); vbox.set_spacing(6)

        hbox = gtk.HBox(); hbox.set_spacing(6)
        label = gtk.Label(_("Lower"))
        entry = gtk.Entry()
        dialog.lower_limit = entry
        entry.set_text(lower);
        hbox.add(label)
        hbox.add(entry)
        hbox.show_all()
        vbox.add(hbox)
        entry.set_activates_default(True)

        hbox = gtk.HBox(); hbox.set_spacing(6)
        label = gtk.Label(_("Upper"))
        entry = gtk.Entry()
        dialog.upper_limit = entry
        entry.set_text(upper);
        hbox.add(label)
        hbox.add(entry)
        hbox.show_all()
        vbox.add(hbox)
        entry.set_activates_default(True)
        
        dialog.vbox.add(create_indenting_container(_("Limits"), vbox))

        dialog.show()
        return dialog

    def __axis_properties_validate(self, dialog, response):
        close_dialog = 1
        rv = None
        if response == gtk.RESPONSE_ACCEPT:
            try:
                pos = float(dialog.position_entry.get_text())
                lower = float(dialog.lower_limit.get_text())
                upper = float(dialog.upper_limit.get_text())
                if lower >= upper:
                    raise ValueError(_("Lower limit is not less than upper limit"))
                rv = pos, lower, upper
            except Exception, ex:
                errdlg = gtk.MessageDialog(
                    flags          = gtk.DIALOG_DESTROY_WITH_PARENT,
                    parent         = dialog,
                    type           = gtk.MESSAGE_ERROR,
                    buttons        = gtk.BUTTONS_OK,
                    message_format = str(ex))
                errdlg.run()
                errdlg.destroy()
                close_dialog = 0
                rv = None
        # maybe close dialog and return
        if close_dialog:
            dialog.destroy()
        return rv

    def __haxis_properties_dialog_response(self, dialog, response):
        values = self.__axis_properties_validate(dialog, response)
        if values is not None:
            pos, lower, upper = values
            self.plot_area.haxis.math_position = pos
            self.hadj.set_range(lower, upper)
    def __vaxis_properties_dialog_response(self, dialog, response):
        values = self.__axis_properties_validate(dialog, response)
        if values is not None:
            pos, lower, upper = values
            self.plot_area.vaxis.math_position = pos
            self.vadj.set_range(lower, upper)

    def create_haxis_properties_dialog(self):
        if self.__haxis_properties_dialog is not None:
            self.__haxis_properties_dialog.present()
            return
        dialog = self.__create_axis_properties_dialog_common(
            _("Horizontal"), "Y", str(self.plot_area.haxis.math_position),
            str(self.hadj.lower), str(self.hadj.upper))
        dialog.connect("response", self.__haxis_properties_dialog_response)
        self.__haxis_properties_dialog = dialog
    def create_vaxis_properties_dialog(self):
        if self.__vaxis_properties_dialog is not None:
            self.__vaxis_properties_dialog.present()
            return
        dialog = self.__create_axis_properties_dialog_common(
            _("Vertical"), "X", str(self.plot_area.vaxis.math_position),
            str(self.vadj.lower), str(self.vadj.upper))
        dialog.connect("response", self.__vaxis_properties_dialog_response)
        self.__vaxis_properties_dialog = dialog

    def __hruler_button_press(self, hruler, event):
        if event.button != 2:
            return 0
        self.create_haxis_properties_dialog()
    def __vruler_button_press(self, vruler, event):
        if event.button != 2:
            return 0
        self.create_vaxis_properties_dialog()


def main():
    window = gtk.Window()
    window.connect("destroy", lambda w: gtk.main_quit())
    window.add(XYGraphView(None).get_widget())
    window.show_all()
    gtk.main()


if __name__ == '__main__':
    main()
